from __future__ import division
from __future__ import absolute_import
# from builtins import object
from past.utils import old_div
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group
import numpy as np
from . import pydart2_api as papi
from .contact import Contact


class CollisionResult(object):
    """
    """

    def __init__(self, _world):
        """
        """
        self.world = _world
        self.id = self.world.id
        self.contacts = list()
        self.contacted_bodies = list()

    def num_contacts(self,):
        return len(self.contacts)

    def num_contacted_bodies(self,):
        return len(self.contacted_bodies)

    def update(self,):
        self.contacts = list()
        self.contacted_bodies = list()

        # contacts
        n = papi.collisionresult__getNumContacts(self.id)
        v = papi.collisionresult__getContacts(self.id, n * 10)
        if n > 0:
            self.contacts = [Contact(self.world, v_i)
                             for v_i in np.split(v, n)]

        # contacted_bodies
        ids = np.array(papi.collisionresult__getCollidingBodyNodes(self.id))
        n = old_div(len(ids), 2)
        if n > 0:
            self.contacted_bodies = [self.world.skeletons[i].bodynodes[j]
                                     for i, j in np.split(ids, n)]

    def __repr__(self,):
        ret = "[CollisionResult: %d contacts %d contacted bodies]" % (
            self.num_contacts(), self.num_contacted_bodies())
        return ret



from __future__ import absolute_import
# from builtins import object
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group
from . import pydart2_api as papi


class Dof(object):
    """
    """
    def __init__(self, _skeleton, _id):
        """
        """
        self.skeleton = _skeleton
        self.id = _id
        self.name = papi.dof__getName(self.wid, self.skid, self.id)
        self.index = self.index_in_skeleton()
        self.joint = None

    @property
    def skel(self):
        return self.skeleton

    @property
    def wid(self):
        return self.skel.world.id

    @property
    def skid(self):
        return self.skel.id

########################################
# Dof::Index functions
    def index_in_skeleton(self, ):
        return papi.dof__getIndexInSkeleton(self.wid,
                                            self.skid,
                                            self.id)

    def index_in_tree(self, ):
        return papi.dof__getIndexInTree(self.wid,
                                        self.skid,
                                        self.id)

    def index_in_joint(self, ):
        return papi.dof__getIndexInJoint(self.wid,
                                         self.skid,
                                         self.id)

    def tree_index(self, ):
        return papi.dof__getTreeIndex(self.wid, self.skid, self.id)

########################################
# Dof::Position functions
    def position(self, ):
        return papi.dof__getPosition(self.wid, self.skid, self.id)

    def set_position(self, _position):
        papi.dof__setPosition(self.wid, self.skid, self.id, _position)

    def initial_position(self, ):
        return papi.dof__getInitialPosition(self.wid, self.skid, self.id)

    def set_initial_position(self, _initial):
        papi.dof__setInitialPosition(self.wid, self.skid, self.id, _initial)

    def has_position_limit(self, ):
        return papi.dof__hasPositionLimit(self.wid, self.skid, self.id)

    def position_lower_limit(self, ):
        return papi.dof__getPositionLowerLimit(self.wid, self.skid, self.id)

    def set_position_lower_limit(self, _limit):
        papi.dof__setPositionLowerLimit(self.wid, self.skid, self.id, _limit)

    def position_upper_limit(self, ):
        return papi.dof__getPositionUpperLimit(self.wid, self.skid, self.id)

    def set_position_upper_limit(self, _limit):
        papi.dof__setPositionUpperLimit(self.wid, self.skid, self.id, _limit)

########################################
# Dof::Velocity functions
    def velocity(self, ):
        return papi.dof__getVelocity(self.wid, self.skid, self.id)

    def set_velocity(self, _velocity):
        papi.dof__setVelocity(self.wid, self.skid, self.id, _velocity)

    def initial_velocity(self, ):
        return papi.dof__getInitialVelocity(self.wid, self.skid, self.id)

    def set_initial_velocity(self, _initial):
        papi.dof__setInitialVelocity(self.wid, self.skid, self.id, _initial)

    def velocity_lower_limit(self, ):
        return papi.dof__getVelocityLowerLimit(self.wid, self.skid, self.id)

    def set_velocity_lower_limit(self, _limit):
        papi.dof__setVelocityLowerLimit(self.wid, self.skid, self.id, _limit)

    def velocity_upper_limit(self, ):
        return papi.dof__getVelocityUpperLimit(self.wid, self.skid, self.id)

    def set_velocity_upper_limit(self, _limit):
        papi.dof__setVelocityUpperLimit(self.wid, self.skid, self.id, _limit)

########################################
# Dof::Passive Force functions
    def spring_stiffness(self, ):
        return papi.dof__getSpringStiffness(self.wid, self.skid, self.id)

    def set_spring_stiffness(self, _k):
        papi.dof__setSpringStiffness(self.wid, self.skid, self.id, _k)

    def rest_position(self, ):
        return papi.dof__getRestPosition(self.wid, self.skid, self.id)

    def set_rest_position(self, _q0):
        papi.dof__setRestPosition(self.wid, self.skid, self.id, _q0)

    def damping_coefficient(self, ):
        return papi.dof__getDampingCoefficient(self.wid, self.skid, self.id)

    def set_damping_coefficient(self, _coeff):
        papi.dof__setDampingCoefficient(self.wid, self.skid, self.id, _coeff)

    def coulomb_friction(self, ):
        return papi.dof__getCoulombFriction(self.wid, self.skid, self.id)

    def set_coulomb_friction(self, _friction):
        papi.dof__setCoulombFriction(self.wid, self.skid, self.id, _friction)

########################################
# Dof::Constraint Impulse functions
    def constraint_impulse(self, ):
        return papi.dof__getConstraintImpulse(self.wid, self.skid, self.id)

    def set_constraint_impulse(self, _friction):
        papi.dof__setConstraintImpulse(self.wid, self.skid, self.id, _friction)

    def __repr__(self):
        return '[Dof(%d): %s]' % (self.id, self.name)



import OpenGL.GL as GL
import OpenGL.GLU as GLU
import OpenGL.GLUT as GLUT
import sys
import numpy as np
from pydart2.gui.opengl.scene import OpenGLScene

# Some api in the chain is translating the keystrokes to this octal string
# so instead of saying: ESCAPE = 27, we use the following.


class GLUTWindow(object):
    def __init__(self, sim, title):
        self.sim = sim
        self.title = title if title is not None else "GLUT Window"
        self.window_size = (1280, 720)
        self.scene = OpenGLScene(*self.window_size)

        self.mouseLastPos = None
        self.is_simulating = False
        self.is_animating = False
        self.frame_index = 0
        self.capture_index = 0

    def initGL(self, w, h):
        self.scene.init()

    def resizeGL(self, w, h):
        self.scene.resize(w, h)

    def drawGL(self, ):
        self.scene.render(self.sim)
        # GLUT.glutSolidSphere(0.3, 20, 20)  # Default object for debugging
        GLUT.glutSwapBuffers()

    # The function called whenever a key is pressed.
    # Note the use of Python tuples to pass in: (key, x, y)
    def keyPressed(self, key, x, y):
        keycode = ord(key)
        key = key.decode('utf-8')
        # print("key = [%s] = [%d]" % (key, ord(key)))

        # n = sim.num_frames()
        if keycode == 27:
            GLUT.glutDestroyWindow(self.window)
            sys.exit()
        elif key == ' ':
            self.is_simulating = not self.is_simulating
            self.is_animating = False
            print("self.is_simulating = %s" % self.is_simulating)
        elif key == 'a':
            self.is_animating = not self.is_animating
            self.is_simulating = False
            print("self.is_animating = %s" % self.is_animating)
        elif key == ']':
            self.frame_index = (self.frame_index + 1) % self.sim.num_frames()
            print("frame = %d/%d" % (self.frame_index, self.sim.num_frames()))
            if hasattr(self.sim, "set_frame"):
                self.sim.set_frame(self.frame_index)
        elif key == '[':
            self.frame_index = (self.frame_index - 1) % self.sim.num_frames()
            print("frame = %d/%d" % (self.frame_index, self.sim.num_frames()))
            if hasattr(self.sim, "set_frame"):
                self.sim.set_frame(self.frame_index)
        elif key == 'c':
            self.capture()

    def mouseFunc(self, button, state, x, y):
        if state == 0:  # Mouse pressed
            self.mouseLastPos = np.array([x, y])
        elif state == 1:
            self.mouseLastPos = None

    def motionFunc(self, x, y):
        dx = x - self.mouseLastPos[0]
        dy = y - self.mouseLastPos[1]
        modifiers = GLUT.glutGetModifiers()
        tb = self.scene.tb
        if modifiers == GLUT.GLUT_ACTIVE_SHIFT:
            tb.zoom_to(dx, -dy)
        elif modifiers == GLUT.GLUT_ACTIVE_CTRL:
            tb.trans_to(dx, -dy)
        else:
            tb.drag_to(x, y, dx, -dy)
        self.mouseLastPos = np.array([x, y])

    def idle(self, ):
        if self.sim is None:
            return
        if self.is_simulating:
            self.sim.step()
            # if self.sim.frame % 10 == 0:
            #     self.capture()
        elif self.is_animating:
            self.frame_index = (self.frame_index + 1) % self.sim.num_frames()
            if hasattr(self.sim, "set_frame"):
                self.sim.set_frame(self.frame_index)

    def renderTimer(self, timer):
        GLUT.glutPostRedisplay()
        GLUT.glutTimerFunc(20, self.renderTimer, 1)

    def capture(self, ):
        print("capture! index = %d" % self.capture_index)
        from PIL import Image
        GL.glPixelStorei(GL.GL_PACK_ALIGNMENT, 1)
        w, h = 1280, 720
        data = GL.glReadPixels(0, 0, w, h, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE)
        img = Image.fromstring("RGBA", (w, h), data)
        img = img.transpose(Image.FLIP_TOP_BOTTOM)
        filename = "./data/captures/capture%04d.png" % self.capture_index
        img.save(filename, 'png')
        self.capture_index += 1

    def run(self, ):
        print("\n")
        print("space bar: simulation on/off")
        print("' ': run/stop simulation")
        print("'a': run/stop animation")
        print("'[' and ']': play one frame backward and forward")

        # Init glut
        GLUT.glutInit(())
        GLUT.glutInitDisplayMode(GLUT.GLUT_RGBA |
                                 GLUT.GLUT_DOUBLE |
                                 GLUT.GLUT_MULTISAMPLE |
                                 GLUT.GLUT_ALPHA |
                                 GLUT.GLUT_DEPTH)
        GLUT.glutInitWindowSize(*self.window_size)
        GLUT.glutInitWindowPosition(0, 0)
        self.window = GLUT.glutCreateWindow(self.title)

        # Init functions
        # glutFullScreen()
        GLUT.glutDisplayFunc(self.drawGL)
        GLUT.glutIdleFunc(self.idle)
        GLUT.glutReshapeFunc(self.resizeGL)
        GLUT.glutKeyboardFunc(self.keyPressed)
        GLUT.glutMouseFunc(self.mouseFunc)
        GLUT.glutMotionFunc(self.motionFunc)
        GLUT.glutTimerFunc(25, self.renderTimer, 1)
        self.initGL(*self.window_size)

        # Run
        GLUT.glutMainLoop()



import unittest
import pydart2 as pydart
from tests.config import DATA_DIR


class TestSkeleton(unittest.TestCase):
    def setUp(self):
        pydart.init(verbose=False)

    def test_atlas_sdf(self):
        world = pydart.world.World(0.001)
        self.assertEqual(world.id, 0)

        skel = world.add_skeleton(
            DATA_DIR + "sdf/atlas/atlas_v3_no_head.sdf")
        self.assertEqual(len(skel.bodynodes), 28)
        self.assertEqual(len(skel.joints), 28)
        self.assertEqual(len(skel.dofs), 33)

    def test_atlas_urdf(self):
        world = pydart.world.World(0.001)
        self.assertEqual(world.id, 0)

        skel = world.add_skeleton(
            DATA_DIR + "sdf/atlas/atlas_v3_no_head.urdf")
        self.assertEqual(len(skel.bodynodes), 34)
        self.assertEqual(len(skel.joints), 34)
        self.assertEqual(len(skel.dofs), 33)

    def test_puppy_sdf(self):
        world = pydart.world.World(0.001)
        self.assertEqual(world.id, 0)

        skel = world.add_skeleton(DATA_DIR + "sdf/puppy.sdf")
        self.assertEqual(len(skel.bodynodes), 15)
        self.assertEqual(len(skel.joints), 15)
        self.assertEqual(len(skel.dofs), 20)



def launch_window(sim, win, default_camera):
    if default_camera is not None:
        win.scene.set_camera(default_camera)
    win.run()


def launch_pyqt4(sim, title=None, default_camera=None):
    from pydart2.gui.pyqt4.window import PyQt4Window
    win = PyQt4Window(sim, title)
    launch_window(sim, win, default_camera)


def launch_pyqt5(sim, title=None, default_camera=None):
    from pydart2.gui.pyqt5.window import PyQt5Window

    win = PyQt5Window(sim, title)
    launch_window(sim, win, default_camera)


def launch_glut(sim, title=None, default_camera=None):
    from pydart2.gui.glut.window import GLUTWindow
    win = GLUTWindow(sim, title)
    launch_window(sim, win, default_camera)


def launch(sim, title=None, default_camera=None):
    """
    default is PyQt5
    """
    # launch_pyqt4(sim, title, default_camera)
    # launch_pyqt5(sim, title, default_camera)
    launch_glut(sim, title, default_camera)



from __future__ import absolute_import
# from builtins import object
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group
from . import pydart2_api as papi


class Contact(object):
    """
    """
    def __init__(self, _world, _state):
        """
        """
        self.world = _world
        self.state = _state
        self.point = _state[:3]
        self.force = _state[3:6]
        self.skel_id1 = int(_state[6])
        self.bodynode_id1 = int(_state[7])
        self.skel_id2 = int(_state[8])
        self.bodynode_id2 = int(_state[9])

        # self.skel_id1, self.bodynode_id1 = _state[6:8]
        # self.skel_id2, self.bodynode_id2 = _state[8:10]
        self.bodynode1 = None
        self.bodynode2 = None
        # print self.skel_id1, self.bodynode_id1,
        # print self.skel_id2, self.bodynode_id2, _state
        if self.skel_id1 >= 0 and self.bodynode_id1 >= 0:
            skel = self.world.skeletons[self.skel_id1]
            self.bodynode1 = skel.bodynodes[self.bodynode_id1]
        if self.skel_id2 >= 0 and self.bodynode_id2 >= 0:
            skel = self.world.skeletons[self.skel_id2]
            self.bodynode2 = skel.bodynodes[self.bodynode_id2]
        # print self.bodynode1, self.bodynode2

    @property
    def p(self, ):
        return self.point

    @property
    def f(self, ):
        return self.force

    def render(self, size, scale):
        papi.collisionresult__renderContact(self.state[:6], size, scale)

    def __repr__(self, ):
        return "[Contact %s %s]" % (self.point, self.force)



# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group


from pydart2.muscle.tendon_unit import MusculoTendonUnit as MTU
from pydart2.muscle.route import Route



from colorama import Fore, Style


def red(txt):
    return Fore.RED + txt + Style.RESET_ALL


def blue(txt):
    return Fore.BLUE + txt + Style.RESET_ALL


def green(txt):
    return Fore.GREEN + txt + Style.RESET_ALL


def yello(txt):
    return Fore.YELLOW + txt + Style.RESET_ALL


def magenta(txt):
    return Fore.MAGENTA + txt + Style.RESET_ALL


def cyan(txt):
    return Fore.CYAN + Style.BRIGHT + txt + Style.RESET_ALL


def bright_red(txt):
    return Fore.RED + Style.BRIGHT + txt + Style.RESET_ALL


def bright_blue(txt):
    return Fore.BLUE + Style.BRIGHT + txt + Style.RESET_ALL


def bright_green(txt):
    return Fore.GREEN + Style.BRIGHT + txt + Style.RESET_ALL


def bright_yello(txt):
    return Fore.YELLOW + Style.BRIGHT + txt + Style.RESET_ALL


def bright_magenta(txt):
    return Fore.MAGENTA + Style.BRIGHT + txt + Style.RESET_ALL


def bright_cyan(txt):
    return Fore.CYAN + Style.BRIGHT + txt + Style.RESET_ALL


def dim_red(txt):
    return Fore.RED + Style.DIM + txt + Style.RESET_ALL


def dim_blue(txt):
    return Fore.BLUE + Style.DIM + txt + Style.RESET_ALL


def dim_green(txt):
    return Fore.GREEN + Style.DIM + txt + Style.RESET_ALL


def dim_yello(txt):
    return Fore.YELLOW + Style.DIM + txt + Style.RESET_ALL


def dim_magenta(txt):
    return Fore.MAGENTA + Style.DIM + txt + Style.RESET_ALL


def dim_cyan(txt):
    return Fore.CYAN + Style.DIM + txt + Style.RESET_ALL



# Author(s): Sehoon Ha <sehoon.ha@gmail.com>
#          : Seungmoon Song <ssm0445@gmail.com>
import numpy as np
from pydart2.utils.misc import S
import pydart2.utils.transformations as trans
from itertools import tee


class AttachmentPoint(object):
    """
    """
    def __init__(self, bodyname, offset):
        self.bodyname = bodyname
        self.skeleton = None
        self.body = None
        self.offset = np.array(offset)

    def is_initialized(self, ):
        return (self.body is not None)

    def initialize(self, skeleton):
        self.skeleton = skeleton
        self.body = skeleton.body(self.bodyname)

    def to_world(self, ):
        return self.body.to_world(self.offset)

    def __str__(self, ):
        return "(%s, %s)" % (self.bodyname, S(self.offset, 3))


class Route(object):
    """
    route = Route([("Upper", [0.0, 0.2, 0.0]), ("Lower", [0.0, 0.2, 0.0])])
    """
    def __init__(self, points=None):
        if points is None:
            self.points = []
        else:
            self.points = [AttachmentPoint(name, offset)
                           for name, offset in points]

        self.world_points = None
        self.world_directions = None
        self.length = None

    def num_points(self, ):
        return len(self.points)

    def __len__(self, ):
        return self.num_points()

    def add_point(self, bodyname, offset):
        pt = AttachmentPoint(bodyname=bodyname,
                             offset=offset)
        self.points.append(pt)

    def initialize_points(self, skeleton):
        for pt in self.points:
            pt.initialize(skeleton)
        self.local_points = [pt.offset for pt in self.points]
        self.bodynodes = [pt.body for pt in self.points]
        self.update_geometry_variables()

    def update_geometry_variables(self, ):
        self.world_points = [pt.to_world() for pt in self.points]
        self.length = 0.0
        self.world_directions = list()

        pt0 = self.world_points[0]
        for pt1 in self.world_points[1:]:
            diff = pt1 - pt0
            length = np.linalg.norm(diff)
            direction = diff / length

            self.length += length
            self.world_directions.append(direction)

            pt0 = pt1

    def local_points_as_pair(self, ):
        "s -> (offset0, offset1), (offset1, offset2), ..."
        a, b = tee(self.local_points)
        next(b, None)
        return zip(a, b)

    def bodynodes_as_pair(self, ):
        "s -> (body0, body1), (body1, body2), ..."
        a, b = tee(self.bodynodes)
        next(b, None)
        return zip(a, b)

    def render_with_ri(self, ri, ):
        if self.num_points() < 2:
            return
        ri.set_line_width(3)
        world_points = [pt.to_world() for pt in self.points]
        ri.render_lines(world_points)

    def __repr__(self, ):
        tokens = [str(pt) for pt in self.points]
        return "[%s: length = %.4f]" % (", ".join(tokens), self.length)



from __future__ import absolute_import
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group
from . import transformations
from . import misc
from . import log
from . import jsonpickle
from . import time_measure
try:
    from . import colored_text
    assert(colored_text)
except Exception:
    print("Error while importing pydart2.utils.colored_text")

assert(transformations)
assert(misc)
assert(log)
assert(jsonpickle)
assert(time_measure)



from __future__ import absolute_import
# from builtins import object
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group
from . import pydart2_api as papi
import numpy as np


class Shape(object):
    """
    """
    def __init__(self, _shapenode):
        """
        """
        self.shapenode = _shapenode
        self.bodynode = _shapenode.bodynode
        self.skeleton = _shapenode.bodynode.skeleton
        self.id = _shapenode.id

    @property
    def skel(self):
        return self.skeleton

    @property
    def wid(self):
        return self.skel.world.id

    @property
    def skid(self):
        return self.skel.id

    @property
    def bid(self):
        return self.bodynode.id

    def build(self):
        pass

    def volume(self, ):
        return papi.shape__getVolume(self.wid, self.skid, self.bid, self.id)

    def shape_type(self, ):
        return self.type_id()
        # type_ = papi.shape__getShapeType(self.wid, self.skid,
        #                                  self.bid, self.id)
        # type_ = 1 if type_ == -1 else type_  # Assume to be ELLIPSOID
        # return type_

    def shape_type_name(self, ):
        return self.type()
        # type_ = self.shape_type()
        # names = ["BOX", "ELLIPSOID", "CYLINDER", "PLANE",
        #          "MESH", "SOFT_MESH", "LINE_SEGMENT"]
        # return names[type_]

    def type(self, ):
        return papi.shape__getType(self.wid, self.skid,
                                   self.bid, self.id)

    def type_id(self, ):
        return papi.shape__getTypeID(self.wid, self.skid,
                                     self.bid, self.id)

    def render(self, ):
        papi.shape__render(self.wid, self.skid, self.bid, self.id)

    def bounding_box(self, ):
        _min = papi.shape__getBoundingBoxMin(self.wid,
                                             self.skid,
                                             self.bid,
                                             self.id)
        _max = papi.shape__getBoundingBoxMax(self.wid,
                                             self.skid,
                                             self.bid,
                                             self.id)
        return (_min, _max)

    def __repr__(self):
        return '[Shape(%d:%d)]' % (self.bid, self.id)


class SphereShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def radius(self, ):
        return papi.sphere_shape__getRadius(self.wid, self.skid, self.bid,
                                            self.id)

    def set_radius(self, _radius):
        return papi.sphere_shape__setRadius(self.wid, self.skid, self.bid,
                                            self.id, _radius)

    def __repr__(self):
        return '[SphereShape(%d:%d)]' % (self.bid, self.id)


class BoxShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def size(self, ):
        return papi.box_shape__getSize(self.wid, self.skid, self.bid, self.id)

    def set_size(self, _size):
        return papi.box_shape__setSize(self.wid, self.skid, self.bid, self.id,
                                       _size)

    def __repr__(self):
        return '[BoxShape(%d:%d)]' % (self.bid, self.id)


class EllipsoidShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def size(self, ):
        return papi.ellipsoid_shape__getSize(self.wid, self.skid,
                                             self.bid, self.id)

    def set_size(self, _size):
        return papi.ellipsoid_shape__setSize(self.wid, self.skid,
                                             self.bid, self.id,
                                             _size)

    def __repr__(self):
        return '[EllipsoidShape(%d:%d)]' % (self.bid, self.id)


class CylinderShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def radius(self, ):
        return papi.cylindershape__getRadius(self.wid, self.skid,
                                             self.bid, self.id)

    def set_radius(self, _radius):
        papi.cylindershape__setRadius(self.wid, self.skid, self.bid,
                                      self.id, _radius)

    def height(self, ):
        return papi.cylindershape__getHeight(self.wid, self.skid,
                                             self.bid, self.id)

    def set_height(self, _height):
        papi.cylindershape__setHeight(self.wid, self.skid, self.bid,
                                      self.id, _height)

    def __repr__(self):
        return '[CylinderShape(%d:%d)]' % (self.bid, self.id)


class CapsuleShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def radius(self, ):
        return papi.capsuleshape__getRadius(self.wid, self.skid,
                                            self.bid, self.id)

    def set_radius(self, radius):
        papi.capsuleshape__setRadius(self.wid, self.skid,
                                     self.bid, self.id, radius)

    def height(self, ):
        return papi.capsuleshape__getHeight(self.wid, self.skid,
                                            self.bid, self.id)

    def set_height(self, height):
        papi.capsuleshape__setHeight(self.wid, self.skid, self.bid,
                                     self.id, height)

    def __repr__(self):
        return '[CapsuleShape(%d:%d)]' % (self.bid, self.id)


class ConeShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def radius(self, ):
        return papi.coneshape__getRadius(self.wid, self.skid, self.bid,
                                         self.id)

    def set_radius(self, radius):
        papi.coneshape__setRadius(self.wid, self.skid, self.bid, self.id,
                                  radius)

    def height(self, ):
        return papi.coneshape__getHeight(self.wid, self.skid, self.bid,
                                         self.id)

    def set_height(self, height):
        papi.coneshape__setHeight(self.wid, self.skid, self.bid, self.id,
                                  height)

    def __repr__(self):
        return '[ConeShape(%d:%d)]' % (self.bid, self.id)


class PlaneShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def normal(self, ):
        return papi.planeshape__getNormal(self.wid, self.skid, self.bid,
                                          self.id)

    def set_normal(self, inv3):
        papi.planeshape__setNormal(self.wid, self.skid, self.bid, self.id,
                                   inv3)

    def offset(self, ):
        return papi.planeshape__getOffset(self.wid, self.skid, self.bid,
                                          self.id)

    def set_offset(self, _offset):
        papi.planeshape__setOffset(self.wid, self.skid, self.bid, self.id,
                                   _offset)

    def __repr__(self):
        return '[PlaneShape(%d:%d)]' % (self.bid, self.id)


class MultiSphereShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def add_sphere(self, r, pos):
        sphere = np.concatenate([[r], pos])
        papi.multisphereshape__addSphere(self.wid, self.skid, self.bid,
                                         self.id, sphere)

    def spheres(self, ):
        n = self.num_spheres()
        data = papi.multisphereshape__getSpheres(self.wid, self.skid,
                                                 self.bid, self.id,
                                                 n * 4)
        spheres = np.split(data, n)
        spheres = [{'r': s[0], 'pos': s[1:]} for s in spheres]
        return spheres

    def num_spheres(self, ):
        return papi.multisphereshape__getNumSpheres(self.wid, self.skid,
                                                    self.bid, self.id)

    def __repr__(self):
        return '[MultiSphereShape(%d:%d)]' % (self.bid, self.id)


class MeshShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def scale(self, ):
        return papi.mesh_shape__getScale(self.wid, self.skid,
                                         self.bid, self.id)

    def path(self, ):
        return papi.mesh_shape__getMeshPath(self.wid, self.skid,
                                            self.bid, self.id)

    def __repr__(self):
        return '[MeshShape(%d:%d)]' % (self.bid, self.id)


class SoftMeshShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def __repr__(self):
        return '[SoftMeshShape(%d:%d)]' % (self.bid, self.id)


class LineSegmentShape(Shape):
    def __init__(self, _shapenode):
        Shape.__init__(self, _shapenode)

    def __repr__(self):
        return '[LineSegmentShape(%d:%d)]' % (self.bid, self.id)


def create_shape(shapenode):
    bodynode = shapenode.bodynode
    skeleton = bodynode.skeleton
    world = skeleton.world

    wid, skid, bid, id = world.id, skeleton.id, bodynode.id, shapenode.id

    type_ = papi.shape__getTypeID(wid, skid, bid, id)
    # type_ = 1 if type_ == -1 else type_  # Assume to be ELLIPSOID
    shape_classes = [SphereShape, BoxShape, EllipsoidShape, CylinderShape,
                     CapsuleShape, ConeShape, PlaneShape, MultiSphereShape,
                     MeshShape, SoftMeshShape, LineSegmentShape]
    if 0 <= type_ < len(shape_classes):
        cls = shape_classes[type_]
        ret = cls(shapenode)
        # print("Shape = %s" % ret)
        return ret
    else:
        print("Invalid type: %d" % type_)
        return None



from __future__ import division
# from builtins import range
from past.utils import old_div
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group

import numpy as np


def S(x, precision=6):
    if x is None:
        return "None"
    fmt = "%%.%df" % precision
    if isinstance(x, float):
        return fmt % x
    ret = "["
    tokens = [fmt % v for v in x]
    ret += ", ".join(tokens)
    ret += "]"
    return ret


def split_by_sizes(array, sizes):
    cum_indices = np.cumsum(sizes)
    return np.split(array, cum_indices[:-1])


def norm2(x):
    return (np.linalg.norm(x) ** 2)


def penalize(x, lo, hi):
    if x < lo:
        return lo - x
    if hi < x:
        return x - hi
    return 0.0


def confine(x, lo, hi):
    if isinstance(x, float):
        return min(max(lo, x), hi)
    else:
        return np.array([confine(v, lo, hi) for v in x])


def vel(h, x0, x1, x2):
    if x0 is not None and x2 is not None:
        return old_div((x2 - x0), (2 * h))
    elif x0 is not None and x2 is None:
        return old_div((x1 - x0), h)
    elif x0 is None and x2 is not None:
        return old_div((x2 - x1), h)
    else:
        return np.zeros(x1.shape)


def acc(h, x0, x1, x2):
    if x0 is not None and x2 is not None:
        return old_div((x2 - 2 * x1 + x0), (h * h))
    else:
        return np.zeros(x1.shape)


def grad(fun, x, h):
    n = len(x)
    g = np.zeros(n)
    for i in range(n):
        dx = np.zeros(n)
        dx[i] = h
        f1 = fun(x - dx)
        f2 = fun(x + dx)
        g[i] = old_div((0.5 * f2 - 0.5 * f1), h)
    return g


def grad4(fun, x, h):
    n = len(x)
    g = np.zeros(n)
    for i in range(n):
        dx = np.zeros(n)
        dx[i] = h
        f0 = fun(x - 2.0 * dx)
        f1 = fun(x - 1.0 * dx)
        f2 = fun(x + 1.0 * dx)
        f3 = fun(x + 2.0 * dx)
        g[i] = old_div((1.0 / 12.0 * f0 - 2.0 / 3.0 * f1 +
                2.0 / 3.0 * f2 - 1.0 / 12.0 * f3), (h))
    return g


def deg2rad(x):
    return x / 180.0 * np.pi


def rad2deg(x):
    return x / np.pi * 180.0



from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from past.utils import old_div
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group
import OpenGL.GL as GL
import OpenGL.GLU as GLU
# import OpenGL.GLUT as GLUT
from PyQt5 import QtGui
from PyQt5 import QtCore
from PyQt5 import QtWidgets
from PyQt5.QtOpenGL import QGLWidget, QGLFormat
from pydart2.gui import trackball
# import time
from pydart2.gui.opengl.scene import OpenGLScene
import numpy as np
# from numpy.linalg import norm


class GLWidget(QtWidgets.QOpenGLWidget):

    def __init__(self, parent=None):
        super(GLWidget, self).__init__(parent)

        fmt = QtGui.QSurfaceFormat()
        # fmt.setSampleBuffers(True)
        fmt.setSamples(4)
        self.setFormat(fmt)

        self.width = 1280
        self.height = 720

        self.lastPos = None
        self.zoom = -1.2

        self.sim = None
        self.viewer = None
        self.captureIndex = 0

        self.scene = OpenGLScene(self.width, self.height, parent)
        self.renderer = self.scene.renderer

        self.lock_camera = False

    def sizeHint(self):
        return QtCore.QSize(self.width, self.height)

    def paintGL(self):
        self.scene.render(self.sim)

    def resizeGL(self, w, h):
        self.scene.resize(w, h)

    def initializeGL(self):
        self.scene.init()

    def set_lock_camera(self, lock=True):
        self.lock_camera = lock

    def mousePressEvent(self, event):
        self.lastPos = event.pos()

        pos = np.array([event.pos().x(), event.pos().y()],
                       dtype=np.float64)
        self.viewer.safe_call_callback("on_mouse_press", pos)

    def mouseReleaseEvent(self, event):
        self.lastPos = None
        self.viewer.safe_call_callback("on_mouse_release")

    def mouseMoveEvent(self, event):
        # (w, h) = (self.width, self.height)
        x = event.x()
        y = event.y()
        dx = event.x() - self.lastPos.x()
        dy = event.y() - self.lastPos.y()

        if not self.lock_camera:
            modifiers = QtWidgets.QApplication.keyboardModifiers()
            if modifiers == QtCore.Qt.ShiftModifier:
                self.scene.tb.zoom_to(dx, -dy)
            elif modifiers == QtCore.Qt.ControlModifier:
                self.scene.tb.trans_to(dx, -dy)
            else:
                self.scene.tb.drag_to(x, y, dx, -dy)

        p0 = np.array([self.lastPos.x(), self.lastPos.y()],
                      dtype=np.float64)
        p1 = np.array([event.pos().x(), event.pos().y()],
                      dtype=np.float64)
        self.viewer.safe_call_callback("on_mouse_move", p0, p1)

        self.lastPos = event.pos()
        # self.paintGL()
        self.update()

    def capture(self, name=None):
        data = GL.glReadPixels(0, 0,
                               self.width, self.height,
                               GL.GL_RGBA,
                               GL.GL_UNSIGNED_BYTE)
        img = QtGui.QImage(data, self.width, self.height,
                           QtGui.QImage.Format_RGBA8888_Premultiplied)
        T = QtGui.QTransform()
        T.scale(1.0, -1.0)
        img = img.transformed(T)

        if name is None:
            name = 'frame'
        dir = self.viewer.capture_dir()
        filename = '%s/%s.%04d.png' % (dir, name, self.captureIndex)
        img.save(filename)
        print('Capture to %s' % filename)
        self.captureIndex += 1



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.8
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_pydart2_api', [dirname(__file__)])
        except ImportError:
            import _pydart2_api
            return _pydart2_api
        if fp is not None:
            try:
                _mod = imp.load_module('_pydart2_api', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _pydart2_api = swig_import_helper()
    del swig_import_helper
else:
    import _pydart2_api
del version_info
try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.


def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr_nondynamic(self, class_type, name, static=1):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    if (not static):
        return object.__getattr__(self, name)
    else:
        raise AttributeError(name)

def _swig_getattr(self, class_type, name):
    return _swig_getattr_nondynamic(self, class_type, name, 0)


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object:
        pass
    _newclass = 0


class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _pydart2_api.delete_SwigPyIterator
    __del__ = lambda self: None

    def value(self) -> "PyObject *":
        return _pydart2_api.SwigPyIterator_value(self)

    def incr(self, n: 'size_t'=1) -> "swig::SwigPyIterator *":
        return _pydart2_api.SwigPyIterator_incr(self, n)

    def decr(self, n: 'size_t'=1) -> "swig::SwigPyIterator *":
        return _pydart2_api.SwigPyIterator_decr(self, n)

    def distance(self, x: 'SwigPyIterator') -> "ptrdiff_t":
        return _pydart2_api.SwigPyIterator_distance(self, x)

    def equal(self, x: 'SwigPyIterator') -> "bool":
        return _pydart2_api.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _pydart2_api.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _pydart2_api.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _pydart2_api.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _pydart2_api.SwigPyIterator_previous(self)

    def advance(self, n: 'ptrdiff_t') -> "swig::SwigPyIterator *":
        return _pydart2_api.SwigPyIterator_advance(self, n)

    def __eq__(self, x: 'SwigPyIterator') -> "bool":
        return _pydart2_api.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: 'SwigPyIterator') -> "bool":
        return _pydart2_api.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: 'ptrdiff_t') -> "swig::SwigPyIterator &":
        return _pydart2_api.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: 'ptrdiff_t') -> "swig::SwigPyIterator &":
        return _pydart2_api.SwigPyIterator___isub__(self, n)

    def __add__(self, n: 'ptrdiff_t') -> "swig::SwigPyIterator *":
        return _pydart2_api.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _pydart2_api.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self
SwigPyIterator_swigregister = _pydart2_api.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)

class IntVector(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, IntVector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, IntVector, name)
    __repr__ = _swig_repr

    def iterator(self) -> "swig::SwigPyIterator *":
        return _pydart2_api.IntVector_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self) -> "bool":
        return _pydart2_api.IntVector___nonzero__(self)

    def __bool__(self) -> "bool":
        return _pydart2_api.IntVector___bool__(self)

    def __len__(self) -> "std::vector< int >::size_type":
        return _pydart2_api.IntVector___len__(self)

    def __getslice__(self, i: 'std::vector< int >::difference_type', j: 'std::vector< int >::difference_type') -> "std::vector< int,std::allocator< int > > *":
        return _pydart2_api.IntVector___getslice__(self, i, j)

    def __setslice__(self, *args) -> "void":
        return _pydart2_api.IntVector___setslice__(self, *args)

    def __delslice__(self, i: 'std::vector< int >::difference_type', j: 'std::vector< int >::difference_type') -> "void":
        return _pydart2_api.IntVector___delslice__(self, i, j)

    def __delitem__(self, *args) -> "void":
        return _pydart2_api.IntVector___delitem__(self, *args)

    def __getitem__(self, *args) -> "std::vector< int >::value_type const &":
        return _pydart2_api.IntVector___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        return _pydart2_api.IntVector___setitem__(self, *args)

    def pop(self) -> "std::vector< int >::value_type":
        return _pydart2_api.IntVector_pop(self)

    def append(self, x: 'std::vector< int >::value_type const &') -> "void":
        return _pydart2_api.IntVector_append(self, x)

    def empty(self) -> "bool":
        return _pydart2_api.IntVector_empty(self)

    def size(self) -> "std::vector< int >::size_type":
        return _pydart2_api.IntVector_size(self)

    def swap(self, v: 'IntVector') -> "void":
        return _pydart2_api.IntVector_swap(self, v)

    def begin(self) -> "std::vector< int >::iterator":
        return _pydart2_api.IntVector_begin(self)

    def end(self) -> "std::vector< int >::iterator":
        return _pydart2_api.IntVector_end(self)

    def rbegin(self) -> "std::vector< int >::reverse_iterator":
        return _pydart2_api.IntVector_rbegin(self)

    def rend(self) -> "std::vector< int >::reverse_iterator":
        return _pydart2_api.IntVector_rend(self)

    def clear(self) -> "void":
        return _pydart2_api.IntVector_clear(self)

    def get_allocator(self) -> "std::vector< int >::allocator_type":
        return _pydart2_api.IntVector_get_allocator(self)

    def pop_back(self) -> "void":
        return _pydart2_api.IntVector_pop_back(self)

    def erase(self, *args) -> "std::vector< int >::iterator":
        return _pydart2_api.IntVector_erase(self, *args)

    def __init__(self, *args):
        this = _pydart2_api.new_IntVector(*args)
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def push_back(self, x: 'std::vector< int >::value_type const &') -> "void":
        return _pydart2_api.IntVector_push_back(self, x)

    def front(self) -> "std::vector< int >::value_type const &":
        return _pydart2_api.IntVector_front(self)

    def back(self) -> "std::vector< int >::value_type const &":
        return _pydart2_api.IntVector_back(self)

    def assign(self, n: 'std::vector< int >::size_type', x: 'std::vector< int >::value_type const &') -> "void":
        return _pydart2_api.IntVector_assign(self, n, x)

    def resize(self, *args) -> "void":
        return _pydart2_api.IntVector_resize(self, *args)

    def insert(self, *args) -> "void":
        return _pydart2_api.IntVector_insert(self, *args)

    def reserve(self, n: 'std::vector< int >::size_type') -> "void":
        return _pydart2_api.IntVector_reserve(self, n)

    def capacity(self) -> "std::vector< int >::size_type":
        return _pydart2_api.IntVector_capacity(self)
    __swig_destroy__ = _pydart2_api.delete_IntVector
    __del__ = lambda self: None
IntVector_swigregister = _pydart2_api.IntVector_swigregister
IntVector_swigregister(IntVector)


def init(verbose: 'bool'=True) -> "void":
    return _pydart2_api.init(verbose)
init = _pydart2_api.init

def destroy() -> "void":
    return _pydart2_api.destroy()
destroy = _pydart2_api.destroy

def setVerbose(verbose: 'bool'=True) -> "void":
    return _pydart2_api.setVerbose(verbose)
setVerbose = _pydart2_api.setVerbose

def getVerbose() -> "bool":
    return _pydart2_api.getVerbose()
getVerbose = _pydart2_api.getVerbose

def marker__getBodyNode(wid: 'int', skid: 'int', mid: 'int') -> "int":
    return _pydart2_api.marker__getBodyNode(wid, skid, mid)
marker__getBodyNode = _pydart2_api.marker__getBodyNode

def marker__getLocalPosition(wid: 'int', skid: 'int', mid: 'int') -> "double [3]":
    return _pydart2_api.marker__getLocalPosition(wid, skid, mid)
marker__getLocalPosition = _pydart2_api.marker__getLocalPosition

def marker__setLocalPosition(wid: 'int', skid: 'int', mid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.marker__setLocalPosition(wid, skid, mid, inv3)
marker__setLocalPosition = _pydart2_api.marker__setLocalPosition

def marker__getWorldPosition(wid: 'int', skid: 'int', mid: 'int') -> "double [3]":
    return _pydart2_api.marker__getWorldPosition(wid, skid, mid)
marker__getWorldPosition = _pydart2_api.marker__getWorldPosition

def marker__render(wid: 'int', skid: 'int', mid: 'int') -> "void":
    return _pydart2_api.marker__render(wid, skid, mid)
marker__render = _pydart2_api.marker__render

def collisionresult__getNumContacts(wid: 'int') -> "int":
    return _pydart2_api.collisionresult__getNumContacts(wid)
collisionresult__getNumContacts = _pydart2_api.collisionresult__getNumContacts

def collisionresult__getContacts(wid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.collisionresult__getContacts(wid, outv)
collisionresult__getContacts = _pydart2_api.collisionresult__getContacts

def collisionresult__getCollidingBodyNodes(wid: 'int') -> "std::vector< int,std::allocator< int > >":
    return _pydart2_api.collisionresult__getCollidingBodyNodes(wid)
collisionresult__getCollidingBodyNodes = _pydart2_api.collisionresult__getCollidingBodyNodes

def collisionresult__renderContact(inv6: 'double [6]', size: 'double', scale: 'double') -> "void":
    return _pydart2_api.collisionresult__renderContact(inv6, size, scale)
collisionresult__renderContact = _pydart2_api.collisionresult__renderContact

def addBallJointConstraint(wid: 'int', skid1: 'int', bid1: 'int', skid2: 'int', bid2: 'int', inv3: 'double [3]') -> "int":
    return _pydart2_api.addBallJointConstraint(wid, skid1, bid1, skid2, bid2, inv3)
addBallJointConstraint = _pydart2_api.addBallJointConstraint

def createWorld(timestep: 'double') -> "int":
    return _pydart2_api.createWorld(timestep)
createWorld = _pydart2_api.createWorld

def createWorldFromSkel(path: 'char const *const') -> "int":
    return _pydart2_api.createWorldFromSkel(path)
createWorldFromSkel = _pydart2_api.createWorldFromSkel

def destroyWorld(wid: 'int') -> "void":
    return _pydart2_api.destroyWorld(wid)
destroyWorld = _pydart2_api.destroyWorld

def world__addSkeleton(wid: 'int', path: 'char const *const') -> "int":
    return _pydart2_api.world__addSkeleton(wid, path)
world__addSkeleton = _pydart2_api.world__addSkeleton

def world__getNumSkeletons(wid: 'int') -> "int":
    return _pydart2_api.world__getNumSkeletons(wid)
world__getNumSkeletons = _pydart2_api.world__getNumSkeletons

def world__reset(wid: 'int') -> "void":
    return _pydart2_api.world__reset(wid)
world__reset = _pydart2_api.world__reset

def world__step(wid: 'int') -> "void":
    return _pydart2_api.world__step(wid)
world__step = _pydart2_api.world__step

def world__checkCollision(wid: 'int') -> "void":
    return _pydart2_api.world__checkCollision(wid)
world__checkCollision = _pydart2_api.world__checkCollision

def world__render(wid: 'int') -> "void":
    return _pydart2_api.world__render(wid)
world__render = _pydart2_api.world__render

def world__setTimeStep(wid: 'int', _timeStep: 'double') -> "void":
    return _pydart2_api.world__setTimeStep(wid, _timeStep)
world__setTimeStep = _pydart2_api.world__setTimeStep

def world__getTimeStep(wid: 'int') -> "double":
    return _pydart2_api.world__getTimeStep(wid)
world__getTimeStep = _pydart2_api.world__getTimeStep

def world__setTime(wid: 'int', _time: 'double') -> "void":
    return _pydart2_api.world__setTime(wid, _time)
world__setTime = _pydart2_api.world__setTime

def world__getTime(wid: 'int') -> "double":
    return _pydart2_api.world__getTime(wid)
world__getTime = _pydart2_api.world__getTime

def world__getSimFrames(wid: 'int') -> "int":
    return _pydart2_api.world__getSimFrames(wid)
world__getSimFrames = _pydart2_api.world__getSimFrames

def world__getIndex(wid: 'int', _index: 'int') -> "int":
    return _pydart2_api.world__getIndex(wid, _index)
world__getIndex = _pydart2_api.world__getIndex

def world__setGravity(wid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.world__setGravity(wid, inv3)
world__setGravity = _pydart2_api.world__setGravity

def world__getGravity(wid: 'int') -> "double [3]":
    return _pydart2_api.world__getGravity(wid)
world__getGravity = _pydart2_api.world__getGravity

def world__setCollisionDetector(wid: 'int', detector_type: 'int') -> "void":
    return _pydart2_api.world__setCollisionDetector(wid, detector_type)
world__setCollisionDetector = _pydart2_api.world__setCollisionDetector

def world__getCollisionDetector(wid: 'int') -> "int":
    return _pydart2_api.world__getCollisionDetector(wid)
world__getCollisionDetector = _pydart2_api.world__getCollisionDetector

def world__removeAllConstraints(wid: 'int') -> "void":
    return _pydart2_api.world__removeAllConstraints(wid)
world__removeAllConstraints = _pydart2_api.world__removeAllConstraints

def skeleton__render(wid: 'int', skid: 'int') -> "void":
    return _pydart2_api.skeleton__render(wid, skid)
skeleton__render = _pydart2_api.skeleton__render

def skeleton__renderWithColor(wid: 'int', skid: 'int', inv4: 'double [4]') -> "void":
    return _pydart2_api.skeleton__renderWithColor(wid, skid, inv4)
skeleton__renderWithColor = _pydart2_api.skeleton__renderWithColor

def skeleton__getName(wid: 'int', skid: 'int') -> "char const *":
    return _pydart2_api.skeleton__getName(wid, skid)
skeleton__getName = _pydart2_api.skeleton__getName

def skeleton__getMass(wid: 'int', skid: 'int') -> "double":
    return _pydart2_api.skeleton__getMass(wid, skid)
skeleton__getMass = _pydart2_api.skeleton__getMass

def skeleton__isMobile(wid: 'int', skid: 'int') -> "bool":
    return _pydart2_api.skeleton__isMobile(wid, skid)
skeleton__isMobile = _pydart2_api.skeleton__isMobile

def skeleton__setMobile(wid: 'int', skid: 'int', mobile: 'bool') -> "void":
    return _pydart2_api.skeleton__setMobile(wid, skid, mobile)
skeleton__setMobile = _pydart2_api.skeleton__setMobile

def skeleton__getSelfCollisionCheck(wid: 'int', skid: 'int') -> "bool":
    return _pydart2_api.skeleton__getSelfCollisionCheck(wid, skid)
skeleton__getSelfCollisionCheck = _pydart2_api.skeleton__getSelfCollisionCheck

def skeleton__setSelfCollisionCheck(wid: 'int', skid: 'int', enable: 'int') -> "void":
    return _pydart2_api.skeleton__setSelfCollisionCheck(wid, skid, enable)
skeleton__setSelfCollisionCheck = _pydart2_api.skeleton__setSelfCollisionCheck

def skeleton__getAdjacentBodyCheck(wid: 'int', skid: 'int') -> "bool":
    return _pydart2_api.skeleton__getAdjacentBodyCheck(wid, skid)
skeleton__getAdjacentBodyCheck = _pydart2_api.skeleton__getAdjacentBodyCheck

def skeleton__setAdjacentBodyCheck(wid: 'int', skid: 'int', enable: 'int') -> "void":
    return _pydart2_api.skeleton__setAdjacentBodyCheck(wid, skid, enable)
skeleton__setAdjacentBodyCheck = _pydart2_api.skeleton__setAdjacentBodyCheck

def skeleton__setRootJointToTransAndEuler(wid: 'int', skid: 'int') -> "void":
    return _pydart2_api.skeleton__setRootJointToTransAndEuler(wid, skid)
skeleton__setRootJointToTransAndEuler = _pydart2_api.skeleton__setRootJointToTransAndEuler

def skeleton__setRootJointToWeld(wid: 'int', skid: 'int') -> "void":
    return _pydart2_api.skeleton__setRootJointToWeld(wid, skid)
skeleton__setRootJointToWeld = _pydart2_api.skeleton__setRootJointToWeld

def skeleton__getNumBodyNodes(wid: 'int', skid: 'int') -> "int":
    return _pydart2_api.skeleton__getNumBodyNodes(wid, skid)
skeleton__getNumBodyNodes = _pydart2_api.skeleton__getNumBodyNodes

def skeleton__getNumJoints(wid: 'int', skid: 'int') -> "int":
    return _pydart2_api.skeleton__getNumJoints(wid, skid)
skeleton__getNumJoints = _pydart2_api.skeleton__getNumJoints

def skeleton__getNumDofs(wid: 'int', skid: 'int') -> "int":
    return _pydart2_api.skeleton__getNumDofs(wid, skid)
skeleton__getNumDofs = _pydart2_api.skeleton__getNumDofs

def skeleton__getNumMarkers(wid: 'int', skid: 'int') -> "int":
    return _pydart2_api.skeleton__getNumMarkers(wid, skid)
skeleton__getNumMarkers = _pydart2_api.skeleton__getNumMarkers

def skeleton__getPositions(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getPositions(wid, skid, outv)
skeleton__getPositions = _pydart2_api.skeleton__getPositions

def skeleton__setPositions(wid: 'int', skid: 'int', inv: 'double *') -> "void":
    return _pydart2_api.skeleton__setPositions(wid, skid, inv)
skeleton__setPositions = _pydart2_api.skeleton__setPositions

def skeleton__getVelocities(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getVelocities(wid, skid, outv)
skeleton__getVelocities = _pydart2_api.skeleton__getVelocities

def skeleton__setVelocities(wid: 'int', skid: 'int', inv: 'double *') -> "void":
    return _pydart2_api.skeleton__setVelocities(wid, skid, inv)
skeleton__setVelocities = _pydart2_api.skeleton__setVelocities

def skeleton__getAccelerations(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getAccelerations(wid, skid, outv)
skeleton__getAccelerations = _pydart2_api.skeleton__getAccelerations

def skeleton__setForces(wid: 'int', skid: 'int', inv: 'double *') -> "void":
    return _pydart2_api.skeleton__setForces(wid, skid, inv)
skeleton__setForces = _pydart2_api.skeleton__setForces

def skeleton__getPositionDifferences(wid: 'int', skid: 'int', inv1: 'double *', inv2: 'double *', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getPositionDifferences(wid, skid, inv1, inv2, outv)
skeleton__getPositionDifferences = _pydart2_api.skeleton__getPositionDifferences

def skeleton__getVelocityDifferences(wid: 'int', skid: 'int', inv1: 'double *', inv2: 'double *', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getVelocityDifferences(wid, skid, inv1, inv2, outv)
skeleton__getVelocityDifferences = _pydart2_api.skeleton__getVelocityDifferences

def skeleton__getPositionLowerLimits(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getPositionLowerLimits(wid, skid, outv)
skeleton__getPositionLowerLimits = _pydart2_api.skeleton__getPositionLowerLimits

def skeleton__getPositionUpperLimits(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getPositionUpperLimits(wid, skid, outv)
skeleton__getPositionUpperLimits = _pydart2_api.skeleton__getPositionUpperLimits

def skeleton__getForceLowerLimits(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getForceLowerLimits(wid, skid, outv)
skeleton__getForceLowerLimits = _pydart2_api.skeleton__getForceLowerLimits

def skeleton__getForceUpperLimits(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getForceUpperLimits(wid, skid, outv)
skeleton__getForceUpperLimits = _pydart2_api.skeleton__getForceUpperLimits

def skeleton__getCOM(wid: 'int', skid: 'int') -> "double [3]":
    return _pydart2_api.skeleton__getCOM(wid, skid)
skeleton__getCOM = _pydart2_api.skeleton__getCOM

def skeleton__getCOMLinearVelocity(wid: 'int', skid: 'int') -> "double [3]":
    return _pydart2_api.skeleton__getCOMLinearVelocity(wid, skid)
skeleton__getCOMLinearVelocity = _pydart2_api.skeleton__getCOMLinearVelocity

def skeleton__getCOMLinearAcceleration(wid: 'int', skid: 'int') -> "double [3]":
    return _pydart2_api.skeleton__getCOMLinearAcceleration(wid, skid)
skeleton__getCOMLinearAcceleration = _pydart2_api.skeleton__getCOMLinearAcceleration

def skeleton__getMassMatrix(wid: 'int', skid: 'int', outm: 'double *') -> "void":
    return _pydart2_api.skeleton__getMassMatrix(wid, skid, outm)
skeleton__getMassMatrix = _pydart2_api.skeleton__getMassMatrix

def skeleton__getCoriolisAndGravityForces(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getCoriolisAndGravityForces(wid, skid, outv)
skeleton__getCoriolisAndGravityForces = _pydart2_api.skeleton__getCoriolisAndGravityForces

def skeleton__getConstraintForces(wid: 'int', skid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.skeleton__getConstraintForces(wid, skid, outv)
skeleton__getConstraintForces = _pydart2_api.skeleton__getConstraintForces

def bodynode__getName(wid: 'int', skid: 'int', bid: 'int') -> "char const *":
    return _pydart2_api.bodynode__getName(wid, skid, bid)
bodynode__getName = _pydart2_api.bodynode__getName

def bodynode__getParentBodyNode(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getParentBodyNode(wid, skid, bid)
bodynode__getParentBodyNode = _pydart2_api.bodynode__getParentBodyNode

def bodynode__getNumChildBodyNodes(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getNumChildBodyNodes(wid, skid, bid)
bodynode__getNumChildBodyNodes = _pydart2_api.bodynode__getNumChildBodyNodes

def bodynode__getChildBodyNode(wid: 'int', skid: 'int', bid: 'int', _index: 'int') -> "int":
    return _pydart2_api.bodynode__getChildBodyNode(wid, skid, bid, _index)
bodynode__getChildBodyNode = _pydart2_api.bodynode__getChildBodyNode

def bodynode__getParentJoint(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getParentJoint(wid, skid, bid)
bodynode__getParentJoint = _pydart2_api.bodynode__getParentJoint

def bodynode__getNumChildJoints(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getNumChildJoints(wid, skid, bid)
bodynode__getNumChildJoints = _pydart2_api.bodynode__getNumChildJoints

def bodynode__getChildJoint(wid: 'int', skid: 'int', bid: 'int', _index: 'int') -> "int":
    return _pydart2_api.bodynode__getChildJoint(wid, skid, bid, _index)
bodynode__getChildJoint = _pydart2_api.bodynode__getChildJoint

def bodynode__getNumDependentDofs(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getNumDependentDofs(wid, skid, bid)
bodynode__getNumDependentDofs = _pydart2_api.bodynode__getNumDependentDofs

def bodynode__getDependentDof(wid: 'int', skid: 'int', bid: 'int', _index: 'int') -> "int":
    return _pydart2_api.bodynode__getDependentDof(wid, skid, bid, _index)
bodynode__getDependentDof = _pydart2_api.bodynode__getDependentDof

def bodynode__getNumShapeNodes(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getNumShapeNodes(wid, skid, bid)
bodynode__getNumShapeNodes = _pydart2_api.bodynode__getNumShapeNodes

def bodynode__getIndexInSkeleton(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getIndexInSkeleton(wid, skid, bid)
bodynode__getIndexInSkeleton = _pydart2_api.bodynode__getIndexInSkeleton

def bodynode__getIndexInTree(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getIndexInTree(wid, skid, bid)
bodynode__getIndexInTree = _pydart2_api.bodynode__getIndexInTree

def bodynode__getTreeIndex(wid: 'int', skid: 'int', bid: 'int') -> "int":
    return _pydart2_api.bodynode__getTreeIndex(wid, skid, bid)
bodynode__getTreeIndex = _pydart2_api.bodynode__getTreeIndex

def bodynode__setGravityMode(wid: 'int', skid: 'int', bid: 'int', _gravityMode: 'bool') -> "void":
    return _pydart2_api.bodynode__setGravityMode(wid, skid, bid, _gravityMode)
bodynode__setGravityMode = _pydart2_api.bodynode__setGravityMode

def bodynode__getGravityMode(wid: 'int', skid: 'int', bid: 'int') -> "bool":
    return _pydart2_api.bodynode__getGravityMode(wid, skid, bid)
bodynode__getGravityMode = _pydart2_api.bodynode__getGravityMode

def bodynode__isCollidable(wid: 'int', skid: 'int', bid: 'int') -> "bool":
    return _pydart2_api.bodynode__isCollidable(wid, skid, bid)
bodynode__isCollidable = _pydart2_api.bodynode__isCollidable

def bodynode__setCollidable(wid: 'int', skid: 'int', bid: 'int', _isCollidable: 'bool') -> "void":
    return _pydart2_api.bodynode__setCollidable(wid, skid, bid, _isCollidable)
bodynode__setCollidable = _pydart2_api.bodynode__setCollidable

def bodynode__getMass(wid: 'int', skid: 'int', bid: 'int') -> "double":
    return _pydart2_api.bodynode__getMass(wid, skid, bid)
bodynode__getMass = _pydart2_api.bodynode__getMass

def bodynode__setMass(wid: 'int', skid: 'int', bid: 'int', mass: 'double') -> "void":
    return _pydart2_api.bodynode__setMass(wid, skid, bid, mass)
bodynode__setMass = _pydart2_api.bodynode__setMass

def bodynode__getInertia(wid: 'int', skid: 'int', bid: 'int') -> "double [3][3]":
    return _pydart2_api.bodynode__getInertia(wid, skid, bid)
bodynode__getInertia = _pydart2_api.bodynode__getInertia

def bodynode__setInertia(wid: 'int', skid: 'int', bid: 'int', inv33: 'double [3][3]') -> "void":
    return _pydart2_api.bodynode__setInertia(wid, skid, bid, inv33)
bodynode__setInertia = _pydart2_api.bodynode__setInertia

def bodynode__getLocalCOM(wid: 'int', skid: 'int', bid: 'int') -> "double [3]":
    return _pydart2_api.bodynode__getLocalCOM(wid, skid, bid)
bodynode__getLocalCOM = _pydart2_api.bodynode__getLocalCOM

def bodynode__getCOM(wid: 'int', skid: 'int', bid: 'int') -> "double [3]":
    return _pydart2_api.bodynode__getCOM(wid, skid, bid)
bodynode__getCOM = _pydart2_api.bodynode__getCOM

def bodynode__getCOMLinearVelocity(wid: 'int', skid: 'int', bid: 'int') -> "double [3]":
    return _pydart2_api.bodynode__getCOMLinearVelocity(wid, skid, bid)
bodynode__getCOMLinearVelocity = _pydart2_api.bodynode__getCOMLinearVelocity

def bodynode__getCOMSpatialVelocity(wid: 'int', skid: 'int', bid: 'int') -> "double [6]":
    return _pydart2_api.bodynode__getCOMSpatialVelocity(wid, skid, bid)
bodynode__getCOMSpatialVelocity = _pydart2_api.bodynode__getCOMSpatialVelocity

def bodynode__getCOMLinearAcceleration(wid: 'int', skid: 'int', bid: 'int') -> "double [3]":
    return _pydart2_api.bodynode__getCOMLinearAcceleration(wid, skid, bid)
bodynode__getCOMLinearAcceleration = _pydart2_api.bodynode__getCOMLinearAcceleration

def bodynode__getCOMSpatialAcceleration(wid: 'int', skid: 'int', bid: 'int') -> "double [6]":
    return _pydart2_api.bodynode__getCOMSpatialAcceleration(wid, skid, bid)
bodynode__getCOMSpatialAcceleration = _pydart2_api.bodynode__getCOMSpatialAcceleration

def bodynode__setFrictionCoeff(wid: 'int', skid: 'int', bid: 'int', _coeff: 'double') -> "void":
    return _pydart2_api.bodynode__setFrictionCoeff(wid, skid, bid, _coeff)
bodynode__setFrictionCoeff = _pydart2_api.bodynode__setFrictionCoeff

def bodynode__getFrictionCoeff(wid: 'int', skid: 'int', bid: 'int') -> "double":
    return _pydart2_api.bodynode__getFrictionCoeff(wid, skid, bid)
bodynode__getFrictionCoeff = _pydart2_api.bodynode__getFrictionCoeff

def bodynode__setRestitutionCoeff(wid: 'int', skid: 'int', bid: 'int', _coeff: 'double') -> "void":
    return _pydart2_api.bodynode__setRestitutionCoeff(wid, skid, bid, _coeff)
bodynode__setRestitutionCoeff = _pydart2_api.bodynode__setRestitutionCoeff

def bodynode__getRestitutionCoeff(wid: 'int', skid: 'int', bid: 'int') -> "double":
    return _pydart2_api.bodynode__getRestitutionCoeff(wid, skid, bid)
bodynode__getRestitutionCoeff = _pydart2_api.bodynode__getRestitutionCoeff

def bodynode__getTransform(wid: 'int', skid: 'int', bid: 'int') -> "double [4][4]":
    return _pydart2_api.bodynode__getTransform(wid, skid, bid)
bodynode__getTransform = _pydart2_api.bodynode__getTransform

def bodynode__getWorldTransform(wid: 'int', skid: 'int', bid: 'int') -> "double [4][4]":
    return _pydart2_api.bodynode__getWorldTransform(wid, skid, bid)
bodynode__getWorldTransform = _pydart2_api.bodynode__getWorldTransform

def bodynode__getRelativeTransform(wid: 'int', skid: 'int', bid: 'int') -> "double [4][4]":
    return _pydart2_api.bodynode__getRelativeTransform(wid, skid, bid)
bodynode__getRelativeTransform = _pydart2_api.bodynode__getRelativeTransform

def bodynode__addExtForce(wid: 'int', skid: 'int', bid: 'int', inv3: 'double [3]', inv3_2: 'double [3]', _isForceLocal: 'bool', _isOffsetLocal: 'bool') -> "void":
    return _pydart2_api.bodynode__addExtForce(wid, skid, bid, inv3, inv3_2, _isForceLocal, _isOffsetLocal)
bodynode__addExtForce = _pydart2_api.bodynode__addExtForce

def bodynode__setExtForce(wid: 'int', skid: 'int', bid: 'int', inv3: 'double [3]', inv3_2: 'double [3]', _isForceLocal: 'bool', _isOffsetLocal: 'bool') -> "void":
    return _pydart2_api.bodynode__setExtForce(wid, skid, bid, inv3, inv3_2, _isForceLocal, _isOffsetLocal)
bodynode__setExtForce = _pydart2_api.bodynode__setExtForce

def bodynode__addExtTorque(wid: 'int', skid: 'int', bid: 'int', inv3: 'double [3]', _isLocal: 'bool') -> "void":
    return _pydart2_api.bodynode__addExtTorque(wid, skid, bid, inv3, _isLocal)
bodynode__addExtTorque = _pydart2_api.bodynode__addExtTorque

def bodynode__setExtTorque(wid: 'int', skid: 'int', bid: 'int', inv3: 'double [3]', _isLocal: 'bool') -> "void":
    return _pydart2_api.bodynode__setExtTorque(wid, skid, bid, inv3, _isLocal)
bodynode__setExtTorque = _pydart2_api.bodynode__setExtTorque

def bodynode__getJacobian(wid: 'int', skid: 'int', bid: 'int', inv3: 'double [3]', outm: 'double *') -> "void":
    return _pydart2_api.bodynode__getJacobian(wid, skid, bid, inv3, outm)
bodynode__getJacobian = _pydart2_api.bodynode__getJacobian

def bodynode__getLinearJacobian(wid: 'int', skid: 'int', bid: 'int', inv3: 'double [3]', outm: 'double *') -> "void":
    return _pydart2_api.bodynode__getLinearJacobian(wid, skid, bid, inv3, outm)
bodynode__getLinearJacobian = _pydart2_api.bodynode__getLinearJacobian

def bodynode__getAngularJacobian(wid: 'int', skid: 'int', bid: 'int', outm: 'double *') -> "void":
    return _pydart2_api.bodynode__getAngularJacobian(wid, skid, bid, outm)
bodynode__getAngularJacobian = _pydart2_api.bodynode__getAngularJacobian

def bodynode__getWorldJacobian(wid: 'int', skid: 'int', bid: 'int', inv3: 'double [3]', outm: 'double *') -> "void":
    return _pydart2_api.bodynode__getWorldJacobian(wid, skid, bid, inv3, outm)
bodynode__getWorldJacobian = _pydart2_api.bodynode__getWorldJacobian

def bodynode__getLinearJacobianDeriv(wid: 'int', skid: 'int', bid: 'int', inv3: 'double [3]', outm: 'double *') -> "void":
    return _pydart2_api.bodynode__getLinearJacobianDeriv(wid, skid, bid, inv3, outm)
bodynode__getLinearJacobianDeriv = _pydart2_api.bodynode__getLinearJacobianDeriv

def bodynode__getAngularJacobianDeriv(wid: 'int', skid: 'int', bid: 'int', outm: 'double *') -> "void":
    return _pydart2_api.bodynode__getAngularJacobianDeriv(wid, skid, bid, outm)
bodynode__getAngularJacobianDeriv = _pydart2_api.bodynode__getAngularJacobianDeriv

def dof__getName(wid: 'int', skid: 'int', dofid: 'int') -> "char const *":
    return _pydart2_api.dof__getName(wid, skid, dofid)
dof__getName = _pydart2_api.dof__getName

def dof__getIndexInSkeleton(wid: 'int', skid: 'int', dofid: 'int') -> "int":
    return _pydart2_api.dof__getIndexInSkeleton(wid, skid, dofid)
dof__getIndexInSkeleton = _pydart2_api.dof__getIndexInSkeleton

def dof__getIndexInTree(wid: 'int', skid: 'int', dofid: 'int') -> "int":
    return _pydart2_api.dof__getIndexInTree(wid, skid, dofid)
dof__getIndexInTree = _pydart2_api.dof__getIndexInTree

def dof__getIndexInJoint(wid: 'int', skid: 'int', dofid: 'int') -> "int":
    return _pydart2_api.dof__getIndexInJoint(wid, skid, dofid)
dof__getIndexInJoint = _pydart2_api.dof__getIndexInJoint

def dof__getTreeIndex(wid: 'int', skid: 'int', dofid: 'int') -> "int":
    return _pydart2_api.dof__getTreeIndex(wid, skid, dofid)
dof__getTreeIndex = _pydart2_api.dof__getTreeIndex

def dof__getPosition(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getPosition(wid, skid, dofid)
dof__getPosition = _pydart2_api.dof__getPosition

def dof__setPosition(wid: 'int', skid: 'int', dofid: 'int', _position: 'double') -> "void":
    return _pydart2_api.dof__setPosition(wid, skid, dofid, _position)
dof__setPosition = _pydart2_api.dof__setPosition

def dof__getInitialPosition(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getInitialPosition(wid, skid, dofid)
dof__getInitialPosition = _pydart2_api.dof__getInitialPosition

def dof__setInitialPosition(wid: 'int', skid: 'int', dofid: 'int', _initial: 'double') -> "void":
    return _pydart2_api.dof__setInitialPosition(wid, skid, dofid, _initial)
dof__setInitialPosition = _pydart2_api.dof__setInitialPosition

def dof__hasPositionLimit(wid: 'int', skid: 'int', dofid: 'int') -> "bool":
    return _pydart2_api.dof__hasPositionLimit(wid, skid, dofid)
dof__hasPositionLimit = _pydart2_api.dof__hasPositionLimit

def dof__getPositionLowerLimit(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getPositionLowerLimit(wid, skid, dofid)
dof__getPositionLowerLimit = _pydart2_api.dof__getPositionLowerLimit

def dof__setPositionLowerLimit(wid: 'int', skid: 'int', dofid: 'int', _limit: 'double') -> "void":
    return _pydart2_api.dof__setPositionLowerLimit(wid, skid, dofid, _limit)
dof__setPositionLowerLimit = _pydart2_api.dof__setPositionLowerLimit

def dof__getPositionUpperLimit(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getPositionUpperLimit(wid, skid, dofid)
dof__getPositionUpperLimit = _pydart2_api.dof__getPositionUpperLimit

def dof__setPositionUpperLimit(wid: 'int', skid: 'int', dofid: 'int', _limit: 'double') -> "void":
    return _pydart2_api.dof__setPositionUpperLimit(wid, skid, dofid, _limit)
dof__setPositionUpperLimit = _pydart2_api.dof__setPositionUpperLimit

def dof__getVelocity(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getVelocity(wid, skid, dofid)
dof__getVelocity = _pydart2_api.dof__getVelocity

def dof__setVelocity(wid: 'int', skid: 'int', dofid: 'int', _velocity: 'double') -> "void":
    return _pydart2_api.dof__setVelocity(wid, skid, dofid, _velocity)
dof__setVelocity = _pydart2_api.dof__setVelocity

def dof__getInitialVelocity(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getInitialVelocity(wid, skid, dofid)
dof__getInitialVelocity = _pydart2_api.dof__getInitialVelocity

def dof__setInitialVelocity(wid: 'int', skid: 'int', dofid: 'int', _initial: 'double') -> "void":
    return _pydart2_api.dof__setInitialVelocity(wid, skid, dofid, _initial)
dof__setInitialVelocity = _pydart2_api.dof__setInitialVelocity

def dof__getVelocityLowerLimit(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getVelocityLowerLimit(wid, skid, dofid)
dof__getVelocityLowerLimit = _pydart2_api.dof__getVelocityLowerLimit

def dof__setVelocityLowerLimit(wid: 'int', skid: 'int', dofid: 'int', _limit: 'double') -> "void":
    return _pydart2_api.dof__setVelocityLowerLimit(wid, skid, dofid, _limit)
dof__setVelocityLowerLimit = _pydart2_api.dof__setVelocityLowerLimit

def dof__getVelocityUpperLimit(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getVelocityUpperLimit(wid, skid, dofid)
dof__getVelocityUpperLimit = _pydart2_api.dof__getVelocityUpperLimit

def dof__setVelocityUpperLimit(wid: 'int', skid: 'int', dofid: 'int', _limit: 'double') -> "void":
    return _pydart2_api.dof__setVelocityUpperLimit(wid, skid, dofid, _limit)
dof__setVelocityUpperLimit = _pydart2_api.dof__setVelocityUpperLimit

def dof__getSpringStiffness(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getSpringStiffness(wid, skid, dofid)
dof__getSpringStiffness = _pydart2_api.dof__getSpringStiffness

def dof__setSpringStiffness(wid: 'int', skid: 'int', dofid: 'int', _k: 'double') -> "void":
    return _pydart2_api.dof__setSpringStiffness(wid, skid, dofid, _k)
dof__setSpringStiffness = _pydart2_api.dof__setSpringStiffness

def dof__getRestPosition(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getRestPosition(wid, skid, dofid)
dof__getRestPosition = _pydart2_api.dof__getRestPosition

def dof__setRestPosition(wid: 'int', skid: 'int', dofid: 'int', _q0: 'double') -> "void":
    return _pydart2_api.dof__setRestPosition(wid, skid, dofid, _q0)
dof__setRestPosition = _pydart2_api.dof__setRestPosition

def dof__getDampingCoefficient(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getDampingCoefficient(wid, skid, dofid)
dof__getDampingCoefficient = _pydart2_api.dof__getDampingCoefficient

def dof__setDampingCoefficient(wid: 'int', skid: 'int', dofid: 'int', _coeff: 'double') -> "void":
    return _pydart2_api.dof__setDampingCoefficient(wid, skid, dofid, _coeff)
dof__setDampingCoefficient = _pydart2_api.dof__setDampingCoefficient

def dof__getCoulombFriction(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getCoulombFriction(wid, skid, dofid)
dof__getCoulombFriction = _pydart2_api.dof__getCoulombFriction

def dof__setCoulombFriction(wid: 'int', skid: 'int', dofid: 'int', _friction: 'double') -> "void":
    return _pydart2_api.dof__setCoulombFriction(wid, skid, dofid, _friction)
dof__setCoulombFriction = _pydart2_api.dof__setCoulombFriction

def dof__getConstraintImpulse(wid: 'int', skid: 'int', dofid: 'int') -> "double":
    return _pydart2_api.dof__getConstraintImpulse(wid, skid, dofid)
dof__getConstraintImpulse = _pydart2_api.dof__getConstraintImpulse

def dof__setConstraintImpulse(wid: 'int', skid: 'int', dofid: 'int', _impulse: 'double') -> "void":
    return _pydart2_api.dof__setConstraintImpulse(wid, skid, dofid, _impulse)
dof__setConstraintImpulse = _pydart2_api.dof__setConstraintImpulse

def joint__getName(wid: 'int', skid: 'int', jid: 'int') -> "char const *":
    return _pydart2_api.joint__getName(wid, skid, jid)
joint__getName = _pydart2_api.joint__getName

def joint__setName(wid: 'int', skid: 'int', jid: 'int', _name: 'char const *', _renameDofs: 'bool') -> "char const *":
    return _pydart2_api.joint__setName(wid, skid, jid, _name, _renameDofs)
joint__setName = _pydart2_api.joint__setName

def joint__isKinematic(wid: 'int', skid: 'int', jid: 'int') -> "bool":
    return _pydart2_api.joint__isKinematic(wid, skid, jid)
joint__isKinematic = _pydart2_api.joint__isKinematic

def joint__isDynamic(wid: 'int', skid: 'int', jid: 'int') -> "bool":
    return _pydart2_api.joint__isDynamic(wid, skid, jid)
joint__isDynamic = _pydart2_api.joint__isDynamic

def joint__getType(wid: 'int', skid: 'int', jid: 'int') -> "char const *":
    return _pydart2_api.joint__getType(wid, skid, jid)
joint__getType = _pydart2_api.joint__getType

def joint__setActuatorType(wid: 'int', skid: 'int', jid: 'int', actuator_type: 'int') -> "void":
    return _pydart2_api.joint__setActuatorType(wid, skid, jid, actuator_type)
joint__setActuatorType = _pydart2_api.joint__setActuatorType

def joint__getActuatorType(wid: 'int', skid: 'int', jid: 'int') -> "int":
    return _pydart2_api.joint__getActuatorType(wid, skid, jid)
joint__getActuatorType = _pydart2_api.joint__getActuatorType

def joint__getParentBodyNode(wid: 'int', skid: 'int', jid: 'int') -> "int":
    return _pydart2_api.joint__getParentBodyNode(wid, skid, jid)
joint__getParentBodyNode = _pydart2_api.joint__getParentBodyNode

def joint__getChildBodyNode(wid: 'int', skid: 'int', jid: 'int') -> "int":
    return _pydart2_api.joint__getChildBodyNode(wid, skid, jid)
joint__getChildBodyNode = _pydart2_api.joint__getChildBodyNode

def joint__setTransformFromParentBodyNode(wid: 'int', skid: 'int', jid: 'int', inv44: 'double [4][4]') -> "void":
    return _pydart2_api.joint__setTransformFromParentBodyNode(wid, skid, jid, inv44)
joint__setTransformFromParentBodyNode = _pydart2_api.joint__setTransformFromParentBodyNode

def joint__setTransformFromChildBodyNode(wid: 'int', skid: 'int', jid: 'int', inv44: 'double [4][4]') -> "void":
    return _pydart2_api.joint__setTransformFromChildBodyNode(wid, skid, jid, inv44)
joint__setTransformFromChildBodyNode = _pydart2_api.joint__setTransformFromChildBodyNode

def joint__getTransformFromParentBodyNode(wid: 'int', skid: 'int', jid: 'int') -> "double [4][4]":
    return _pydart2_api.joint__getTransformFromParentBodyNode(wid, skid, jid)
joint__getTransformFromParentBodyNode = _pydart2_api.joint__getTransformFromParentBodyNode

def joint__getTransformFromChildBodyNode(wid: 'int', skid: 'int', jid: 'int') -> "double [4][4]":
    return _pydart2_api.joint__getTransformFromChildBodyNode(wid, skid, jid)
joint__getTransformFromChildBodyNode = _pydart2_api.joint__getTransformFromChildBodyNode

def joint__setPositionLimitEnforced(wid: 'int', skid: 'int', jid: 'int', _isPositionLimitEnforced: 'bool') -> "void":
    return _pydart2_api.joint__setPositionLimitEnforced(wid, skid, jid, _isPositionLimitEnforced)
joint__setPositionLimitEnforced = _pydart2_api.joint__setPositionLimitEnforced

def joint__isPositionLimitEnforced(wid: 'int', skid: 'int', jid: 'int') -> "bool":
    return _pydart2_api.joint__isPositionLimitEnforced(wid, skid, jid)
joint__isPositionLimitEnforced = _pydart2_api.joint__isPositionLimitEnforced

def joint__hasPositionLimit(wid: 'int', skid: 'int', jid: 'int', _index: 'int') -> "bool":
    return _pydart2_api.joint__hasPositionLimit(wid, skid, jid, _index)
joint__hasPositionLimit = _pydart2_api.joint__hasPositionLimit

def joint__getPositionLowerLimit(wid: 'int', skid: 'int', jid: 'int', _index: 'int') -> "double":
    return _pydart2_api.joint__getPositionLowerLimit(wid, skid, jid, _index)
joint__getPositionLowerLimit = _pydart2_api.joint__getPositionLowerLimit

def joint__setPositionLowerLimit(wid: 'int', skid: 'int', jid: 'int', _index: 'int', _position: 'double') -> "void":
    return _pydart2_api.joint__setPositionLowerLimit(wid, skid, jid, _index, _position)
joint__setPositionLowerLimit = _pydart2_api.joint__setPositionLowerLimit

def joint__getPositionUpperLimit(wid: 'int', skid: 'int', jid: 'int', _index: 'int') -> "double":
    return _pydart2_api.joint__getPositionUpperLimit(wid, skid, jid, _index)
joint__getPositionUpperLimit = _pydart2_api.joint__getPositionUpperLimit

def joint__setPositionUpperLimit(wid: 'int', skid: 'int', jid: 'int', _index: 'int', _position: 'double') -> "void":
    return _pydart2_api.joint__setPositionUpperLimit(wid, skid, jid, _index, _position)
joint__setPositionUpperLimit = _pydart2_api.joint__setPositionUpperLimit

def joint__getDof(wid: 'int', skid: 'int', jid: 'int', _index: 'int') -> "int":
    return _pydart2_api.joint__getDof(wid, skid, jid, _index)
joint__getDof = _pydart2_api.joint__getDof

def joint__getNumDofs(wid: 'int', skid: 'int', jid: 'int') -> "int":
    return _pydart2_api.joint__getNumDofs(wid, skid, jid)
joint__getNumDofs = _pydart2_api.joint__getNumDofs

def joint__getSpringStiffness(wid: 'int', skid: 'int', jid: 'int', _index: 'int') -> "double":
    return _pydart2_api.joint__getSpringStiffness(wid, skid, jid, _index)
joint__getSpringStiffness = _pydart2_api.joint__getSpringStiffness

def joint__setSpringStiffness(wid: 'int', skid: 'int', jid: 'int', _index: 'int', _k: 'double') -> "void":
    return _pydart2_api.joint__setSpringStiffness(wid, skid, jid, _index, _k)
joint__setSpringStiffness = _pydart2_api.joint__setSpringStiffness

def joint__getRestPosition(wid: 'int', skid: 'int', jid: 'int', _index: 'int') -> "double":
    return _pydart2_api.joint__getRestPosition(wid, skid, jid, _index)
joint__getRestPosition = _pydart2_api.joint__getRestPosition

def joint__setRestPosition(wid: 'int', skid: 'int', jid: 'int', _index: 'int', _q0: 'double') -> "void":
    return _pydart2_api.joint__setRestPosition(wid, skid, jid, _index, _q0)
joint__setRestPosition = _pydart2_api.joint__setRestPosition

def joint__getDampingCoefficient(wid: 'int', skid: 'int', jid: 'int', _index: 'int') -> "double":
    return _pydart2_api.joint__getDampingCoefficient(wid, skid, jid, _index)
joint__getDampingCoefficient = _pydart2_api.joint__getDampingCoefficient

def joint__setDampingCoefficient(wid: 'int', skid: 'int', jid: 'int', _index: 'int', _coeff: 'double') -> "void":
    return _pydart2_api.joint__setDampingCoefficient(wid, skid, jid, _index, _coeff)
joint__setDampingCoefficient = _pydart2_api.joint__setDampingCoefficient

def joint__getCoulombFriction(wid: 'int', skid: 'int', jid: 'int', _index: 'int') -> "double":
    return _pydart2_api.joint__getCoulombFriction(wid, skid, jid, _index)
joint__getCoulombFriction = _pydart2_api.joint__getCoulombFriction

def joint__setCoulombFriction(wid: 'int', skid: 'int', jid: 'int', _index: 'int', _friction: 'double') -> "void":
    return _pydart2_api.joint__setCoulombFriction(wid, skid, jid, _index, _friction)
joint__setCoulombFriction = _pydart2_api.joint__setCoulombFriction

def revolute_joint__getAxis(wid: 'int', skid: 'int', jid: 'int') -> "double [3]":
    return _pydart2_api.revolute_joint__getAxis(wid, skid, jid)
revolute_joint__getAxis = _pydart2_api.revolute_joint__getAxis

def revolute_joint__setAxis(wid: 'int', skid: 'int', jid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.revolute_joint__setAxis(wid, skid, jid, inv3)
revolute_joint__setAxis = _pydart2_api.revolute_joint__setAxis

def prismatic_joint__getAxis(wid: 'int', skid: 'int', jid: 'int') -> "double [3]":
    return _pydart2_api.prismatic_joint__getAxis(wid, skid, jid)
prismatic_joint__getAxis = _pydart2_api.prismatic_joint__getAxis

def prismatic_joint__setAxis(wid: 'int', skid: 'int', jid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.prismatic_joint__setAxis(wid, skid, jid, inv3)
prismatic_joint__setAxis = _pydart2_api.prismatic_joint__setAxis

def universal_joint__getAxis1(wid: 'int', skid: 'int', jid: 'int') -> "double [3]":
    return _pydart2_api.universal_joint__getAxis1(wid, skid, jid)
universal_joint__getAxis1 = _pydart2_api.universal_joint__getAxis1

def universal_joint__setAxis1(wid: 'int', skid: 'int', jid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.universal_joint__setAxis1(wid, skid, jid, inv3)
universal_joint__setAxis1 = _pydart2_api.universal_joint__setAxis1

def universal_joint__getAxis2(wid: 'int', skid: 'int', jid: 'int') -> "double [3]":
    return _pydart2_api.universal_joint__getAxis2(wid, skid, jid)
universal_joint__getAxis2 = _pydart2_api.universal_joint__getAxis2

def universal_joint__setAxis2(wid: 'int', skid: 'int', jid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.universal_joint__setAxis2(wid, skid, jid, inv3)
universal_joint__setAxis2 = _pydart2_api.universal_joint__setAxis2

def euler_joint__getAxisOrder(wid: 'int', skid: 'int', jid: 'int') -> "char const *":
    return _pydart2_api.euler_joint__getAxisOrder(wid, skid, jid)
euler_joint__getAxisOrder = _pydart2_api.euler_joint__getAxisOrder

def euler_joint__setAxisOrder(wid: 'int', skid: 'int', jid: 'int', axisorder: 'char const *') -> "void":
    return _pydart2_api.euler_joint__setAxisOrder(wid, skid, jid, axisorder)
euler_joint__setAxisOrder = _pydart2_api.euler_joint__setAxisOrder

def shapenode__getOffset(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [3]":
    return _pydart2_api.shapenode__getOffset(wid, skid, bid, sid)
shapenode__getOffset = _pydart2_api.shapenode__getOffset

def shapenode__setOffset(wid: 'int', skid: 'int', bid: 'int', sid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.shapenode__setOffset(wid, skid, bid, sid, inv3)
shapenode__setOffset = _pydart2_api.shapenode__setOffset

def shapenode__getRelativeTransform(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [4][4]":
    return _pydart2_api.shapenode__getRelativeTransform(wid, skid, bid, sid)
shapenode__getRelativeTransform = _pydart2_api.shapenode__getRelativeTransform

def shapenode__setRelativeTransform(wid: 'int', skid: 'int', bid: 'int', sid: 'int', inv44: 'double [4][4]') -> "void":
    return _pydart2_api.shapenode__setRelativeTransform(wid, skid, bid, sid, inv44)
shapenode__setRelativeTransform = _pydart2_api.shapenode__setRelativeTransform

def shapenode__hasVisualAspect(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "bool":
    return _pydart2_api.shapenode__hasVisualAspect(wid, skid, bid, sid)
shapenode__hasVisualAspect = _pydart2_api.shapenode__hasVisualAspect

def shapenode__hasCollisionAspect(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "bool":
    return _pydart2_api.shapenode__hasCollisionAspect(wid, skid, bid, sid)
shapenode__hasCollisionAspect = _pydart2_api.shapenode__hasCollisionAspect

def shapenode__getVisualAspectRGBA(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [4]":
    return _pydart2_api.shapenode__getVisualAspectRGBA(wid, skid, bid, sid)
shapenode__getVisualAspectRGBA = _pydart2_api.shapenode__getVisualAspectRGBA

def shapenode__setVisualAspectRGBA(wid: 'int', skid: 'int', bid: 'int', sid: 'int', inv4: 'double [4]') -> "void":
    return _pydart2_api.shapenode__setVisualAspectRGBA(wid, skid, bid, sid, inv4)
shapenode__setVisualAspectRGBA = _pydart2_api.shapenode__setVisualAspectRGBA

def shape__getVolume(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.shape__getVolume(wid, skid, bid, sid)
shape__getVolume = _pydart2_api.shape__getVolume

def shape__getType(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "char const *":
    return _pydart2_api.shape__getType(wid, skid, bid, sid)
shape__getType = _pydart2_api.shape__getType

def shape__getTypeID(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "int":
    return _pydart2_api.shape__getTypeID(wid, skid, bid, sid)
shape__getTypeID = _pydart2_api.shape__getTypeID

def shape__render(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "void":
    return _pydart2_api.shape__render(wid, skid, bid, sid)
shape__render = _pydart2_api.shape__render

def shape__getBoundingBoxMin(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [3]":
    return _pydart2_api.shape__getBoundingBoxMin(wid, skid, bid, sid)
shape__getBoundingBoxMin = _pydart2_api.shape__getBoundingBoxMin

def shape__getBoundingBoxMax(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [3]":
    return _pydart2_api.shape__getBoundingBoxMax(wid, skid, bid, sid)
shape__getBoundingBoxMax = _pydart2_api.shape__getBoundingBoxMax

def sphere_shape__getRadius(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.sphere_shape__getRadius(wid, skid, bid, sid)
sphere_shape__getRadius = _pydart2_api.sphere_shape__getRadius

def sphere_shape__setRadius(wid: 'int', skid: 'int', bid: 'int', sid: 'int', radius: 'double') -> "void":
    return _pydart2_api.sphere_shape__setRadius(wid, skid, bid, sid, radius)
sphere_shape__setRadius = _pydart2_api.sphere_shape__setRadius

def box_shape__getSize(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [3]":
    return _pydart2_api.box_shape__getSize(wid, skid, bid, sid)
box_shape__getSize = _pydart2_api.box_shape__getSize

def box_shape__setSize(wid: 'int', skid: 'int', bid: 'int', sid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.box_shape__setSize(wid, skid, bid, sid, inv3)
box_shape__setSize = _pydart2_api.box_shape__setSize

def ellipsoid_shape__getSize(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [3]":
    return _pydart2_api.ellipsoid_shape__getSize(wid, skid, bid, sid)
ellipsoid_shape__getSize = _pydart2_api.ellipsoid_shape__getSize

def ellipsoid_shape__setSize(wid: 'int', skid: 'int', bid: 'int', sid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.ellipsoid_shape__setSize(wid, skid, bid, sid, inv3)
ellipsoid_shape__setSize = _pydart2_api.ellipsoid_shape__setSize

def cylindershape__getRadius(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.cylindershape__getRadius(wid, skid, bid, sid)
cylindershape__getRadius = _pydart2_api.cylindershape__getRadius

def cylindershape__setRadius(wid: 'int', skid: 'int', bid: 'int', sid: 'int', _radius: 'double') -> "void":
    return _pydart2_api.cylindershape__setRadius(wid, skid, bid, sid, _radius)
cylindershape__setRadius = _pydart2_api.cylindershape__setRadius

def cylindershape__getHeight(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.cylindershape__getHeight(wid, skid, bid, sid)
cylindershape__getHeight = _pydart2_api.cylindershape__getHeight

def cylindershape__setHeight(wid: 'int', skid: 'int', bid: 'int', sid: 'int', _height: 'double') -> "void":
    return _pydart2_api.cylindershape__setHeight(wid, skid, bid, sid, _height)
cylindershape__setHeight = _pydart2_api.cylindershape__setHeight

def capsuleshape__getRadius(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.capsuleshape__getRadius(wid, skid, bid, sid)
capsuleshape__getRadius = _pydart2_api.capsuleshape__getRadius

def capsuleshape__setRadius(wid: 'int', skid: 'int', bid: 'int', sid: 'int', radius: 'double') -> "void":
    return _pydart2_api.capsuleshape__setRadius(wid, skid, bid, sid, radius)
capsuleshape__setRadius = _pydart2_api.capsuleshape__setRadius

def capsuleshape__getHeight(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.capsuleshape__getHeight(wid, skid, bid, sid)
capsuleshape__getHeight = _pydart2_api.capsuleshape__getHeight

def capsuleshape__setHeight(wid: 'int', skid: 'int', bid: 'int', sid: 'int', height: 'double') -> "void":
    return _pydart2_api.capsuleshape__setHeight(wid, skid, bid, sid, height)
capsuleshape__setHeight = _pydart2_api.capsuleshape__setHeight

def coneshape__getRadius(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.coneshape__getRadius(wid, skid, bid, sid)
coneshape__getRadius = _pydart2_api.coneshape__getRadius

def coneshape__setRadius(wid: 'int', skid: 'int', bid: 'int', sid: 'int', radius: 'double') -> "void":
    return _pydart2_api.coneshape__setRadius(wid, skid, bid, sid, radius)
coneshape__setRadius = _pydart2_api.coneshape__setRadius

def coneshape__getHeight(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.coneshape__getHeight(wid, skid, bid, sid)
coneshape__getHeight = _pydart2_api.coneshape__getHeight

def coneshape__setHeight(wid: 'int', skid: 'int', bid: 'int', sid: 'int', height: 'double') -> "void":
    return _pydart2_api.coneshape__setHeight(wid, skid, bid, sid, height)
coneshape__setHeight = _pydart2_api.coneshape__setHeight

def planeshape__getNormal(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [3]":
    return _pydart2_api.planeshape__getNormal(wid, skid, bid, sid)
planeshape__getNormal = _pydart2_api.planeshape__getNormal

def planeshape__setNormal(wid: 'int', skid: 'int', bid: 'int', sid: 'int', inv3: 'double [3]') -> "void":
    return _pydart2_api.planeshape__setNormal(wid, skid, bid, sid, inv3)
planeshape__setNormal = _pydart2_api.planeshape__setNormal

def planeshape__getOffset(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double":
    return _pydart2_api.planeshape__getOffset(wid, skid, bid, sid)
planeshape__getOffset = _pydart2_api.planeshape__getOffset

def planeshape__setOffset(wid: 'int', skid: 'int', bid: 'int', sid: 'int', _offset: 'double') -> "void":
    return _pydart2_api.planeshape__setOffset(wid, skid, bid, sid, _offset)
planeshape__setOffset = _pydart2_api.planeshape__setOffset

def multisphereshape__addSphere(wid: 'int', skid: 'int', bid: 'int', sid: 'int', inv4: 'double [4]') -> "void":
    return _pydart2_api.multisphereshape__addSphere(wid, skid, bid, sid, inv4)
multisphereshape__addSphere = _pydart2_api.multisphereshape__addSphere

def multisphereshape__getSpheres(wid: 'int', skid: 'int', bid: 'int', sid: 'int', outv: 'double *') -> "int":
    return _pydart2_api.multisphereshape__getSpheres(wid, skid, bid, sid, outv)
multisphereshape__getSpheres = _pydart2_api.multisphereshape__getSpheres

def multisphereshape__getNumSpheres(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "int":
    return _pydart2_api.multisphereshape__getNumSpheres(wid, skid, bid, sid)
multisphereshape__getNumSpheres = _pydart2_api.multisphereshape__getNumSpheres

def mesh_shape__getScale(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "double [3]":
    return _pydart2_api.mesh_shape__getScale(wid, skid, bid, sid)
mesh_shape__getScale = _pydart2_api.mesh_shape__getScale

def mesh_shape__getMeshPath(wid: 'int', skid: 'int', bid: 'int', sid: 'int') -> "char const *":
    return _pydart2_api.mesh_shape__getMeshPath(wid, skid, bid, sid)
mesh_shape__getMeshPath = _pydart2_api.mesh_shape__getMeshPath
# This file is compatible with both classic and new-style classes.





from PyQt5 import QtCore
from PyQt5 import QtGui


class SidePanel(object):

    def __init__(self, width=300):
        self.width = width
        self.widgets = dict()

        self.layout = list()  # layout of widgets
        self.line = list()  # temporary buffer
        self.vbox = None

    def register_widget(self, widget, name, next_line):
        self.widgets[name] = widget
        self.line.append(widget)
        if next_line:
            self.process_line()
            self.line = list()

    def process_line(self,):
        if len(self.line) == 0:
            return
        self.layout.append(self.line)

    def push_line(self,):
        self.process_line()
        self.line = list()

    def build_layout(self,):
        self.process_line()  # Push if not processed yet
        self.vbox = QtGui.QVBoxLayout()
        for i, line in enumerate(self.layout):
            if len(line) == 1:
                self.vbox.addWidget(line[0])
            elif len(line) > 1:
                hbox = QtGui.QHBoxLayout()
                for w in line:
                    hbox.addWidget(w)
                self.vbox.addLayout(hbox)
        self.vbox.addStretch(1)
        return self.vbox

    def add_label(self, name, label=None, next_line=True):
        label = name if label is None else label
        w = QtGui.QLabel(label)
        self.register_widget(w, name, next_line)
        return w

    def add_push_button(self, name, callback=None, label=None, next_line=True):
        label = name if label is None else label
        w = QtGui.QPushButton(label)
        if callback is not None:
            QtCore.QObject.connect(w, QtCore.SIGNAL('clicked()'), callback)
        self.register_widget(w, name, next_line)
        return w

    def add_toggle_button(self, name, default=True,
                          callback=None, label=None,
                          next_line=True):
        w = self.add_push_button(name,
                                 callback=callback,
                                 label=label,
                                 next_line=next_line)
        w.setCheckable(True)
        if default is not None:
            w.setChecked(default)
        return w

    def add_checkbox(self, name, default=True,
                     callback=None, label=None,
                     next_line=True):
        label = name if label is None else label
        w = QtGui.QCheckBox(label)
        self.register_widget(w, name, next_line)

        if callback is not None:
            QtCore.QObject.connect(w, QtCore.SIGNAL('clicked()'), callback)
        if default is not None:
            w.setChecked(default)
        return w

    def add_combobox(self, name, items, default=0,
                     label=True,
                     callback=None, next_line=True):
        if label:
            self.add_label(label if isinstance(label, str)
                           else name, next_line=False)

        w = QtGui.QComboBox()
        w.addItems(items)
        w.setCurrentIndex(default)
        if callback is not None:
            QtCore.QObject.connect(w,
                                   QtCore.SIGNAL('currentIndexChanged(int)'),
                                   callback)
        self.register_widget(w, name, next_line)
        return w

    def add_spinbox(self, name, min, max, default=None, step=None,
                    label=True,
                    prefix=None, suffix=None, callback=None, next_line=True):
        if label:
            self.add_label(label if isinstance(label, str)
                           else name, next_line=False)
        w = QtGui.QSpinBox()
        w.setMinimum(min)
        w.setMaximum(max)
        if default is not None:
            w.setValue(default)
        if step is not None:
            w.setSingleStep(step)
        if prefix is not None:
            w.setPrefix(prefix)
        if suffix is not None:
            w.setSuffix(suffix)
        if callback is not None:
            QtCore.QObject.connect(w,
                                   QtCore.SIGNAL('valueChanged(int)'),
                                   callback)
        self.register_widget(w, name, next_line)
        return w

    def add_double_spinbox(self, name, min, max, default=None, step=None,
                           label=True,
                           prefix=None, suffix=None, callback=None,
                           next_line=True):
        if label:
            self.add_label(label if isinstance(label, str)
                           else name, next_line=False)
        w = QtGui.QDoubleSpinBox()
        w.setMinimum(min)
        w.setMaximum(max)
        if default is not None:
            w.setValue(default)
        if step is not None:
            w.setSingleStep(step)
        if prefix is not None:
            w.setPrefix(prefix)
        if suffix is not None:
            w.setSuffix(suffix)
        if callback is not None:
            QtCore.QObject.connect(w,
                                   QtCore.SIGNAL('valueChanged(double)'),
                                   callback)
        self.register_widget(w, name, next_line)
        return w

    def set_text(self, name, label):
        self.widgets[name].setText(label)

    def value(self, name):
        return self.widgets[name].value()

    def set_value(self, name, value):
        self.widgets[name].setValue(value)

    def is_checked(self, name):
        return self.widgets[name].isChecked()

    def set_checked(self, name, value):
        self.widgets[name].setChecked(value)

    def set_enabled(self, name, enabled=True):
        self.widgets[name].setEnabled(enabled)



# Author(s): Seungmoon Song <ssm0445@gmail.com>
import numpy as np


def fn_inv_f_vce0(f_vce0, K, N):
    if f_vce0 <= 1:
        v_ce0 = (f_vce0 - 1) / (K * f_vce0 + 1)
    elif f_vce0 > 1 and f_vce0 <= N:
        temp = (f_vce0 - N) / (f_vce0 - N + 1)
        v_ce0 = (temp + 1) / (1 - 7.56 * K * temp)
    else:  # elif f_vce0 > N:
        v_ce0 = .01 * (f_vce0 - N) + 1

    return v_ce0


def fn_f_lce0(l_ce0, w, c):
    f_lce0 = np.exp(c * np.abs((l_ce0 - 1) / (w))**3)
    return f_lce0

# f_p0 is for both f_se0 and f_pe0


def fn_f_p0(l0, e_ref):
    if l0 > 1:
        f_p0 = ((l0 - 1) / (e_ref))**2
    else:
        f_p0 = 0

    return f_p0

# for both f_be0


def fn_f_p0_ext(l0, e_ref, e_ref2):
    if l0 < e_ref2:
        f_p0 = ((l0 - e_ref2) / (e_ref))**2
    else:
        f_p0 = 0

    return f_p0



# Author(s): Sehoon Ha <sehoon.ha@gmail.com>
import numpy as np
from pydart2.skeleton import Skeleton


class MusculoSkeleton(Skeleton):
    """
    world = pydart2.world.World(0.001)
    world.add_skeleton("data/skeleton_file", MusculoSkeleton)
    """

    def __init__(self, _world, _filename=None,
                 _id=None, _friction=None, ):
        Skeleton.__init__(self, _world, _filename, _id, _friction)
        self.muscles = list()

    def num_muscles(self, ):
        return len(self.muscles)

    def add_muscle(self, mtu, route):
        route.initialize_points(self)
        route.update_geometry_variables()
        self.muscles.append((mtu, route))

    def update(self, ):
        for mtu in self.tendon_units:
            mtu.update()

    def update_route_geometry(self, ):
        for _, route in self.muscles:
            route.update_geometry_variables()

    def reset_muscles(self, ):
        for (mtu, route) in self.muscles:
            mtu.reset()

    def stimulate(self, stimulation):
        self.update_route_geometry()
        for S, (mtu, route) in zip(stimulation, self.muscles):
            mtu.update(S, route.length)

    def apply_muscle_forces(self, ):
        for (mtu, route) in self.muscles:
            f_mtu = mtu.F_mtu
            for (o0, o1), (b0, b1), u in zip(route.local_points_as_pair(),
                                             route.bodynodes_as_pair(),
                                             route.world_directions):
                if b0 == b1:
                    continue
                F_ext = f_mtu * u
                b0.add_ext_force(F_ext, o0)
                b1.add_ext_force(-F_ext, o1)

    def render_with_ri(self, ri):
        r = np.array([1.0, 0.0, 0.0])
        b = np.array([0.0, 0.0, 1.0])
        for mtu, route in self.muscles:
            w = mtu.a
            color = (1 - w) * b + w * r
            ri.set_color(*color)
            route.render_with_ri(ri)

    def __repr__(self):
        args = (self.id, self.name, self.num_muscles())
        return '[MusculoSkeleton(%d): %s # Muscles = %d]' % args



from __future__ import print_function
from __future__ import absolute_import
import sys

from . import pydart2_api as papi
from . import world
from . import constraints
from . import utils
from .world import World
from .skel_vector import SkelVector

assert utils
assert World
assert SkelVector

assert world
assert constraints

# import os.path

from . import gui

# try:
#     from . import gui
#     assert gui
# except Exception:
#     e = sys.exc_info()[1]
#     print("-" * 40)
#     print("Error while importing pydart2.gui")
#     print(e)
#     print("-" * 40)

__version__ = (0, 5, 0)


def init(verbose=True):
    papi.init(verbose)



import OpenGL.GL as GL
import OpenGL.GLU as GLU
import OpenGL.GLUT as GLUT

from pydart2.gui.opengl.renderer import Renderer
from pydart2.gui.trackball import Trackball


class OpenGLScene(object):
    def __init__(self, width, height, window=None):
        self.width = width
        self.height = height
        self.window = window

        self.renderer = Renderer()

        self.cameras = list()
        self.tb = None
        self.init_cameras()

    def init(self, ):
        GL.glDisable(GL.GL_CULL_FACE)
        GL.glEnable(GL.GL_DEPTH_TEST)

        GL.glDepthFunc(GL.GL_LEQUAL)
        GL.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST)

        GL.glEnable(GL.GL_LINE_SMOOTH)
        GL.glHint(GL.GL_LINE_SMOOTH_HINT, GL.GL_NICEST)
        # GlEnable(GL.GL_POLYGON_SMOOTH)
        GL.glHint(GL.GL_POLYGON_SMOOTH_HINT, GL.GL_NICEST)

        GL.glEnable(GL.GL_DITHER)
        GL.glShadeModel(GL.GL_SMOOTH)
        GL.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST)

        GL.glClearColor(1.0, 1.0, 1.0, 1.0)
        GL.glClear(GL.GL_COLOR_BUFFER_BIT)

        GL.glEnable(GL.GL_DEPTH_TEST)
        GL.glDepthFunc(GL.GL_LEQUAL)
        GL.glDisable(GL.GL_CULL_FACE)
        GL.glEnable(GL.GL_NORMALIZE)

        GL.glColorMaterial(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE)
        GL.glEnable(GL.GL_COLOR_MATERIAL)

        GL.glEnable(GL.GL_BLEND)
        GL.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA)
        GL.glEnable(GL.GL_MULTISAMPLE)
        # GLUT.glutSetOption(GLUT.GLUT_MULTISAMPLE, 4)

        # glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA)

        ambient = [0.2, 0.2, 0.2, 1.0]
        diffuse = [0.6, 0.6, 0.6, 1.0]
        front_mat_shininess = [60.0]
        front_mat_specular = [0.2, 0.2, 0.2, 1.0]
        front_mat_diffuse = [0.5, 0.28, 0.38, 1.0]
        lmodel_ambient = [0.2, 0.2, 0.2, 1.0]
        lmodel_twoside = [GL.GL_FALSE]

        # position = [1.0, 1.0, 1.0, 0.0]
        # position1 = [-1.0, 1.0, 0.0, 0.0]

        position = [1.0, 1.0, 0.0, 0.0]
        position1 = [-1.0, 0.0, 0.0, 0.0]

        GL.glEnable(GL.GL_LIGHT0)
        GL.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, ambient)
        GL.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse)
        GL.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, position)

        GL.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient)
        GL.glLightModelfv(GL.GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside)

        GL.glEnable(GL.GL_LIGHT1)
        # glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT, ambient)
        GL.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, diffuse)
        GL.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, position1)
        GL.glEnable(GL.GL_LIGHTING)

        GL.glEnable(GL.GL_COLOR_MATERIAL)
        GL.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SHININESS,
                        front_mat_shininess)
        GL.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR,
                        front_mat_specular)
        GL.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE,
                        front_mat_diffuse)

    def resize(self, w, h):
        (self.width, self.height) = (w, h)
        GL.glViewport(0, 0, w, h)
        GL.glMatrixMode(GL.GL_PROJECTION)
        GL.glLoadIdentity()

        GLU.gluPerspective(45.0, float(w) / float(h), 0.01, 100.0)
        GL.glMatrixMode(GL.GL_MODELVIEW)
        GL.glLoadIdentity()

    def render(self, sim=None):
        GL.glEnable(GL.GL_DEPTH_TEST)
        GL.glClearColor(0.98, 0.98, 0.98, 0.0)
        GL.glClearColor(1.0, 1.0, 1.0, 1.0)
        GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)

        GL.glLoadIdentity()
        # glTranslate(0.0, -0.2, self.zoom)  # Camera
        GL.glTranslate(*self.tb.trans)
        GL.glMultMatrixf(self.tb.matrix)

        if sim is None:
            return

        if hasattr(sim, "render"):
            sim.render()

        self.renderer.enable("COLOR_MATERIAL")
        if hasattr(sim, "render_with_ri"):
            sim.render_with_ri(self.renderer)

        self.enable2D()
        if hasattr(sim, "draw_with_ri"):
            sim.draw_with_ri(self.renderer)
            self.renderer.draw_text([-100, -100], "")
        self.disable2D()

    def enable2D(self):
        w, h = self.width, self.height
        GL.glMatrixMode(GL.GL_PROJECTION)
        GL.glLoadIdentity()
        GL.glDisable(GL.GL_LIGHTING | GL.GL_DEPTH_TEST)
        GL.glDepthMask(0)
        GL.glOrtho(0, w, h, 0, -1, 1)
        GL.glViewport(0, 0, w, h)
        GL.glMatrixMode(GL.GL_MODELVIEW)
        GL.glLoadIdentity()

    def disable2D(self):
        w, h = self.width, self.height
        GL.glMatrixMode(GL.GL_PROJECTION)
        GL.glLoadIdentity()

        GL.glEnable(GL.GL_DEPTH_TEST | GL.GL_LIGHTING)
        GL.glDepthMask(1)
        GLU.gluPerspective(45.0, float(w) / float(h), 0.01, 100.0)

        GL.glViewport(0, 0, w, h)
        GL.glMatrixMode(GL.GL_MODELVIEW)
        GL.glLoadIdentity()

    def init_cameras(self,):
        self.cameras = list()
        self.add_camera(
            Trackball(
                rot=[-0.152, 0.045, -0.002, 0.987],
                trans=[0.050, 0.210, -2.500]),
            "Camera Y up")
        self.add_camera(
            Trackball(
                rot=[0.535, 0.284, 0.376, 0.701], trans=[0.10, 0.02, -2.770]),
            "Camera Z up")
        self.set_camera(0)

    def num_cameras(self,):
        return len(self.cameras)

    def replace_camera(self, idx, trackball):
        if idx >= self.num_cameras():
            return False
        self.cameras[idx] = trackball
        return True

    def add_camera(self, trackball, name):
        self.cameras.append(trackball)
        if self.window is not None:
            # Need to pass self because glwidget is not inited yet
            self.window.add_camera_event(self, trackball, name)

    def set_camera(self, camera_index):
        print("set_camera: %d" % camera_index)
        self.tb = self.cameras[camera_index]



from __future__ import absolute_import
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group

from pydart2.gui import opengl
assert(opengl)

try:
    from pydart2.gui import glut
    assert(glut)
except Exception:
    print("[Warn] fail to load GLUT -- check the installation")

# try:
#     from pydart2.gui import pyqt4
#     assert(pyqt4)
# except:
#     print("[Warn] fail to load PyQt4 -- check the installation")

try:
    from pydart2.gui import pyqt5
    assert(pyqt5)
except:
    print("[Warn] fail to load PyQt5 -- check the installation")

try:
    from pydart2.gui import viewer
    assert(viewer)
except:
    print("[Warn] fail to load viewer")


# assert(pydart2.gui.opengl)
# assert(pydart2.gui.pyqt4)

# from pydart2.gui import viewer
# from pydart2.gui import side_panel
# assert viewer
# assert side_panel



import time
from functools import partial


def decorator_time_measure(func, text_decor=None):
    def time_func_wrapper(self, *args, **kwargs):
        import time
        tic = time.time()
        ret = func(self, *args, **kwargs)
        toc = time.time()
        txt = "[%s] toc-tic = %.8fs" % (func.__name__, toc - tic)
        if text_decor is not None:
            txt = text_decor(txt)
        if hasattr(self, "logger"):
            self.logger.info(txt)
        else:
            print(txt)
        return ret
    return time_func_wrapper


def decorator_time_measure_colored(text_decor):
    return partial(decorator_time_measure, text_decor=text_decor)


class TimeMeasure(object):
    def __init__(self, logger=None, name=None, color=None):
        self.logger = logger
        self.name = name
        self.color = color

    def __enter__(self, ):
        self.tic = time.time()

    def __exit__(self, type, value, traceback):
        self.toc = time.time()
        txt = "" if self.name is None else self.name
        txt += "toc-tic = %.8f" % (self.toc - self.tic)
        if self.color:
            txt = self.color(txt)
        if self.logger is not None:
            self.logger.info(txt)
        else:
            print(txt)



from setuptools import setup
from setuptools import find_packages
from distutils.core import Extension
from sys import platform as _platform
import sys
import glob

DIR = 'pydart2/'

CXX_FLAGS = '-Wall -msse2 -fPIC -std=c++11 -Xlinker -rpath /usr/local/lib '
CXX_FLAGS += '-O3 -DNDEBUG -shared '
CXX_FLAGS += '-g -fno-omit-frame-pointer -fno-inline-functions '
CXX_FLAGS += '-fno-optimize-sibling-calls '

# CXX_FLAGS = '-fopenmp -Wall -Wextra -fPIC -std=c++11 '
# CXX_FLAGS = '-O3 -DNDEBUG'

# CXX_FLAGS += '-fno-stack-protector '
# CXX_FLAGS += '-DPY_VERSION_HEX=0x03000000'

include_dirs = list()
include_dirs += ['/usr/include']
include_dirs += ['/usr/include/eigen3']

# Fetch python version
python_version = sys.version_info
python_major_version = python_version[0]
python_minor_version = python_version[1]

current_python = "python%d.%d" % (python_major_version, python_minor_version)
print("current_python = %s" % (current_python))
# include_dirs += ['/usr/include/python2.7']
# include_dirs += ['/usr/local/include/python2.7']
include_dirs += ['/usr/include/%s' % current_python]
include_dirs += ['/usr/local/include/%s' % current_python]
include_dirs += ['/usr/include/bullet']
include_dirs += ['/usr/local/include']
include_dirs += ['/usr/local/include/eigen3']
include_dirs += ['/usr/local/include/bullet']
include_dirs += ['/usr/local/Cellar/urdfdom_headers/0.2.3/include']

try:
    import numpy
    NP_DIRS = [numpy.get_include()]
    print("numpy.get_include() = %s" % numpy.get_include())
except:
    NP_DIRS = list()
    NP_DIRS += ['/usr/local/lib/%s/dist-packages/numpy/core/include/' %
                current_python]
    NP_DIRS += ['/usr/lib/%s/dist-packages/numpy/core/include/' %
                current_python]
for d in NP_DIRS:
    print("numpy_include_dirs = %s" % d)
include_dirs += NP_DIRS

libraries = list()
libraries += ['dart', 'dart-gui']
libraries += ['dart-optimizer-nlopt',
              'dart-planning', 'dart-utils', 'dart-utils-urdf']
# libraries += [current_python]
if _platform == "linux" or _platform == "linux2":
    libraries += ['GL', 'glut', 'Xmu', 'Xi']
    CXX_FLAGS += '-fno-inline-functions-called-once'
elif _platform == "darwin":
    CXX_FLAGS += '-framework Cocoa '
    CXX_FLAGS += '-framework OpenGL '
    CXX_FLAGS += '-framework GLUT '

    # libraries += ['GLUT', 'Cocoa', 'OpenGL']
libraries += ['BulletDynamics', 'BulletCollision',
              'LinearMath', 'BulletSoftBody']

swig_opts = ['-c++']
if python_major_version == 3:
    swig_opts.append('-py3')
print("swig_opts: %s" % (str(swig_opts)))

sources = glob.glob(DIR + "/*.cpp")
sources.append(DIR + "pydart2_api.i")
sources = [f for f in sources if "wrap" not in f]  # Safe guard

print("source files:")
for source_file in sources:
    print("    > %s" % source_file)

print("depend files:")
depends = glob.glob(DIR + "/*.h")
depends.append(DIR + "numpy.i")
for depend_file in depends:
    print("    > %s" % depend_file)

MANIFEST_in = "\n".join(["include %s" % f for f in depends])
print("Please check MANIFEST.in is:")
print("============================")
print(MANIFEST_in)
print("============================")

pydart2_api = Extension('_pydart2_api',
                        define_macros=[('MAJOR_VERSION', '1'),
                                       ('MINOR_VERSION', '0')],
                        include_dirs=include_dirs,
                        libraries=libraries,
                        library_dirs=['/usr/local/lib'],
                        extra_compile_args=CXX_FLAGS.split(),
                        swig_opts=swig_opts,
                        sources=sources,
                        depends=depends)

requires = ['numpy', 'PyOpenGL',
            # 'PyOpenGL_accelerate',
            'future', 'six']

setup(name='pydart2',
      version='0.6.5',
      description='Python Interface for DART Simulator',
      url='https://github.com/sehoonha/pydart2',
      author='Sehoon Ha',
      author_email='sehoon.ha@gmail.com',
      license='BSD',
      install_requires=requires,
      keywords=['physics', 'robotics', 'simulation'],
      classifiers=['Development Status :: 2 - Pre-Alpha',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: POSIX :: Linux',
                   'Intended Audience :: Science/Research',
                   "Programming Language :: Python",
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.4',
                   'Topic :: Games/Entertainment :: Simulation'],
      packages=find_packages(),
      ext_modules=[pydart2_api])



from __future__ import absolute_import

try:
    from pydart2.gui.pyqt5 import window
    from pydart2.gui.pyqt5 import side_panel

    assert window
    assert side_panel
except Exception:
    print("failed to load pyqt5")
